#!/usr/bin/env python
# Created by Pearu Peterson, June 2003
""" Test functions for interpolate.fitpack2 module
"""
__usage__ = """
Build interpolate:
  python setup_interpolate.py build
Run tests if scipy is installed:
  python -c 'import scipy;scipy.interpolate.test(<level>)'
Run tests if interpolate is not installed:
  python tests/test_fitpack.py [<level>]
"""
#import libwadpy

import sys
from scipy_test.testing import *
set_package_path()
from interpolate.fitpack2 import UnivariateSpline,LSQUnivariateSpline,\
     InterpolatedUnivariateSpline
from interpolate.fitpack2 import LSQBivariateSpline, SmoothBivariateSpline
del sys.path[0]

import unittest

class test_UnivariateSpline(ScipyTestCase):
    def check_linear_constant(self):
        x = [1,2,3]
        y = [3,3,3]
        lut = UnivariateSpline(x,y,k=1)
        assert_array_almost_equal(lut.get_knots(),[1,3])
        assert_array_almost_equal(lut.get_coeffs(),[3,3])
        assert_almost_equal(lut.get_residual(),0.0)
        assert_array_almost_equal(lut([1,1.5,2]),[3,3,3])

    def check_linear_1d(self):
        x = [1,2,3]
        y = [0,2,4]
        lut = UnivariateSpline(x,y,k=1)
        assert_array_almost_equal(lut.get_knots(),[1,3])
        assert_array_almost_equal(lut.get_coeffs(),[0,4])
        assert_almost_equal(lut.get_residual(),0.0)
        assert_array_almost_equal(lut([1,1.5,2]),[0,1,2])

class test_LSQBivariateSpline(ScipyTestCase):
    def check_linear_constant(self):
        x = [1,1,1,2,2,2,3,3,3]
        y = [1,2,3,1,2,3,1,2,3]
        z = [3,3,3,3,3,3,3,3,3]
        s = 0.1
        tx = [1+s,3-s]
        ty = [1+s,3-s]
        lut = LSQBivariateSpline(x,y,z,tx,ty,kx=1,ky=1)
        #print lut.get_knots()
        #print lut.get_coeffs()
        #print lut.get_residual()

class test_SmoothBivariateSpline(ScipyTestCase):
    def check_linear_constant(self):
        x = [1,1,1,2,2,2,3,3,3]
        y = [1,2,3,1,2,3,1,2,3]
        z = [3,3,3,3,3,3,3,3,3]
        lut = SmoothBivariateSpline(x,y,z,kx=1,ky=1)
        assert_array_almost_equal(lut.get_knots(),([1,1,3,3],[1,1,3,3]))
        assert_array_almost_equal(lut.get_coeffs(),[3,3,3,3])
        assert_almost_equal(lut.get_residual(),0.0)
        assert_array_almost_equal(lut([1,1.5,2],[1,1.5]),[[3,3],[3,3],[3,3]])

    def check_linear_1d(self):
        x = [1,1,1,2,2,2,3,3,3]
        y = [1,2,3,1,2,3,1,2,3]
        z = [0,0,0,2,2,2,4,4,4]
        lut = SmoothBivariateSpline(x,y,z,kx=1,ky=1)
        assert_array_almost_equal(lut.get_knots(),([1,1,3,3],[1,1,3,3]))
        assert_array_almost_equal(lut.get_coeffs(),[0,0,4,4])
        assert_almost_equal(lut.get_residual(),0.0)
        assert_array_almost_equal(lut([1,1.5,2],[1,1.5]),[[0,0],[1,1],[2,2]])

if __name__ == "__main__":
    ScipyTest('interpolate.fitpack').run()



#!/usr/bin/env python

import os, sys
from scipy_distutils.misc_util import get_path, default_config_dict

def configuration(parent_package='',parent_path=None):
    package = 'gplt'
    local_path = get_path(__name__,parent_path)
    config = default_config_dict(package,parent_package)

    if sys.platform == 'win32':
        data = ['wgnuplot.exe', 'gnuplot_helper.exe']
        data = [os.path.join(local_path,x) for x in data]
        config['data_files'].append((os.path.join(parent_package,'gplt'),data))
    return config

if __name__ == '__main__':
    from scipy_distutils.core import setup
    setup(**configuration())




import unittest
import sys
from scipy_test.testing import *
set_package_path()
import scipy_base;reload(scipy_base)
from scipy_base import *
del sys.path[0]
       
class test_isscalar(unittest.TestCase):
    def check_basic(self):
        assert(isscalar(3))
        assert(not isscalar([3]))
        assert(not isscalar((3,)))
        assert(isscalar(3j))
        assert(isscalar(10L))
        assert(isscalar(4.0))

class test_real(unittest.TestCase):
    def check_real(self):
        y = rand(10,)
        assert_array_equal(y,real(y))

    def check_cmplx(self):
        y = rand(10,)+1j*rand(10,)
        assert_array_equal(y.real,real(y))

class test_imag(unittest.TestCase):
    def check_real(self):
        y = rand(10,)
        assert_array_equal(0,imag(y))

    def check_cmplx(self):
        y = rand(10,)+1j*rand(10,)
        assert_array_equal(y.imag,imag(y))

class test_iscomplex(unittest.TestCase):
    def check_fail(self):
        z = array([-1,0,1])
        res = iscomplex(z)
        assert(not sometrue(res))
    def check_pass(self):
        z = array([-1j,1,0])
        res = iscomplex(z)
        assert_array_equal(res,[1,0,0])

class test_isreal(unittest.TestCase):
    def check_pass(self):
        z = array([-1,0,1j])
        res = isreal(z)
        assert_array_equal(res,[1,1,0])
    def check_fail(self):
        z = array([-1j,1,0])
        res = isreal(z)
        assert_array_equal(res,[0,1,1])

class test_iscomplexobj(unittest.TestCase):
    def check_basic(self):
        z = array([-1,0,1])
        assert(not iscomplexobj(z))
        z = array([-1j,0,-1])
        assert(iscomplexobj(z))

class test_isrealobj(unittest.TestCase):
    def check_basic(self):
        z = array([-1,0,1])
        assert(isrealobj(z))
        z = array([-1j,0,-1])
        assert(not isrealobj(z))

class test_isnan(unittest.TestCase):
    def check_goodvalues(self):
        z = array((-1.,0.,1.))
        res = isnan(z) == 0
        assert(alltrue(res))            
    def check_posinf(self): 
        assert(isnan(array((1.,))/0.) == 0)
    def check_neginf(self): 
        assert(isnan(array((-1.,))/0.) == 0)
    def check_ind(self): 
        assert(isnan(array((0.,))/0.) == 1)
    #def check_qnan(self):             log(-1) return pi*j now
    #    assert(isnan(log(-1.)) == 1)
    def check_integer(self):
        assert(isnan(1) == 0)
    def check_complex(self):
        assert(isnan(1+1j) == 0)
    def check_complex1(self):
        assert(isnan(array(0+0j)/0.) == 1)
                
class test_isfinite(unittest.TestCase):
    def check_goodvalues(self):
        z = array((-1.,0.,1.))
        res = isfinite(z) == 1
        assert(alltrue(res))            
    def check_posinf(self): 
        assert(isfinite(array((1.,))/0.) == 0)
    def check_neginf(self): 
        assert(isfinite(array((-1.,))/0.) == 0)
    def check_ind(self): 
        assert(isfinite(array((0.,))/0.) == 0)
    #def check_qnan(self): 
    #    assert(isfinite(log(-1.)) == 0)
    def check_integer(self):
        assert(isfinite(1) == 1)
    def check_complex(self):
        assert(isfinite(1+1j) == 1)
    def check_complex1(self):
        assert(isfinite(array(1+1j)/0.) == 0)
        
class test_isinf(unittest.TestCase):
    def check_goodvalues(self):
        z = array((-1.,0.,1.))
        res = isinf(z) == 0
        assert(alltrue(res))            
    def check_posinf(self): 
        assert(isinf(array((1.,))/0.) == 1)
    def check_posinf_scalar(self): 
        assert(isinf(array(1.,)/0.) == 1)
    def check_neginf(self): 
        assert(isinf(array((-1.,))/0.) == 1)
    def check_neginf_scalar(self): 
        assert(isinf(array(-1.)/0.) == 1)
    def check_ind(self): 
        assert(isinf(array((0.,))/0.) == 0)
    #def check_qnan(self): 
    #    assert(isinf(log(-1.)) == 0)
    #    assert(isnan(log(-1.)) == 1)

class test_isposinf(unittest.TestCase):
    def check_generic(self):
        vals = isposinf(array((-1.,0,1))/0.)
        assert(vals[0] == 0)
        assert(vals[1] == 0)
        assert(vals[2] == 1)

class test_isneginf(unittest.TestCase):
    def check_generic(self):
        vals = isneginf(array((-1.,0,1))/0.)
        assert(vals[0] == 1)
        assert(vals[1] == 0)
        assert(vals[2] == 0)

class test_nan_to_num(unittest.TestCase):
    def check_generic(self):
        vals = nan_to_num(array((-1.,0,1))/0.)
        assert(vals[0] < -1e10 and isfinite(vals[0]))
        assert(vals[1] == 0)
        assert(vals[2] > 1e10 and isfinite(vals[2]))
    def check_integer(self):
        vals = nan_to_num(1)
        assert(vals == 1)
    def check_complex_good(self):
        vals = nan_to_num(1+1j)
        assert(vals == 1+1j)
    def check_complex_bad(self):
        v = 1+1j
        v += array(0+1.j)/0.
        vals = nan_to_num(v)
        # !! This is actually (unexpectedly) zero
        assert(vals.imag > 1e10 and isfinite(vals))
    def check_complex_bad2(self):
        v = 1+1j
        v += array(-1+1.j)/0.
        vals = nan_to_num(v)
        assert(isfinite(vals))    
        #assert(vals.imag > 1e10  and isfinite(vals))    
        # !! This is actually (unexpectedly) positive
        # !! inf.  Comment out for now, and see if it
        # !! changes
        #assert(vals.real < -1e10 and isfinite(vals))    


class test_real_if_close(unittest.TestCase):
    def check_basic(self):
        a = rand(10)
        b = real_if_close(a+1e-15j)
        assert(isrealobj(b))
        assert_array_equal(a,b)
        b = real_if_close(a+1e-7j)
        assert(iscomplexobj(b))
        b = real_if_close(a+1e-7j,tol=1e-6)
        assert(isrealobj(b))

if __name__ == "__main__":
    ScipyTest('scipy_base.type_check').run()



# Copyright (c) 1996, 1997, The Regents of the University of California.
# All rights reserved.  See Legal.htm for full text and disclaimer.

from Numeric import *
from scipy_base.fastumath import *
import NarPlotter
from MLab import rand
from graph3d import *
from surface import *

def paws ( ) :
    i = raw_input ("Type in any string to continue; ^C to return to prompt. ")
    return
 
print "This is a test of the Python interface to the Limeil Lab graphics"
print "package, Narcisse. You  need Narcisse to be running. Fire it up by"
print "doing setenv PORT_SERVEUR 0, typing  /dist/basis/Narcisse/bin/Narcisse &,"
print "and then do another senetv PORT_SERVEUR to the port number which"
print "appears in the Narcisse GUI."
 
paws ()
 
vsf = 0.
c = 1
s = 1000.
kmax = 25
lmax = 35
# The following computations define an interesting 3d surface.

xr = multiply.outer (
   arange (1, kmax + 1, typecode = Float), ones (lmax, Float))
yr = multiply.outer (
   ones (kmax, Float), arange (1, lmax + 1, typecode = Float))
zt = 5. + xr + .2 * rand (kmax, lmax)   # ranf (xr)
rt = 100. + yr + .2 * rand (kmax, lmax)   # ranf (yr)
z = s * (rt + zt)
z = z + .02 * z * rand (kmax, lmax)   # ranf (z)
ut = rt/sqrt (rt ** 2 + zt ** 2)
vt = zt/sqrt (rt ** 2 + zt ** 2)
ireg =  multiply.outer ( ones (kmax), ones (lmax))
ireg [0:1, 0:lmax]=0
ireg [0:kmax, 0:1]=0
ireg [1:15, 7:12]=2
ireg [1:15, 12:lmax]=3
ireg [3:7, 3:7]=0
freg=ireg + .2 * (1. - rand (kmax, lmax))  # ranf (ireg))
freg=array (freg, Float)
#rt [4:6, 4:6] = -1.e8
z [3:10, 3:12] = z [3:10, 3:12] * .9
z [5, 5] = z [5, 5] * .9
z [17:22, 15:18] = z [17:22, 15:18] * 1.2
z [16, 16] = z [16, 16] * 1.1
 
s1 = Surface (x = zt, y = rt, z = freg, mask = "max")
g1 = Graph3d (s1, titles = "Rotating mesh", theta = 45., phi = 5., roll = 0.0)
g1.plot ( )
for i in range (6) :
   g1.quick_plot (roll = (i + 1) * 22.5)
paws ()
g1.quick_plot (surface = 1, opt_3d = "i3", titles = ["(1) contour lines" ,
        "Different options of viewing mesh"], text = "Region 2",
        text_color = "black", text_size = 20, text_pos = [.20, .35])
paws ( )
g1.quick_plot (surface = 1, opt_3d = "s3", titles = ["(2) smooth coloring",
        "Different options of viewing mesh"])
paws ( )
g1.quick_plot (surface = 1, opt_3d = "w3", titles = ["(3) colored mesh",
        "Different options of viewing mesh"])
paws ( )
s2 = Surface (x = zt + vt, y = rt + vt, z = freg + ut,
              mask = "sort", opt_3d = "w3")
g1 = Graph3d ( [s1, s2], connect = 1, text = " ", roll = 0.0,
               titles = "Mesh follows velocity field one time step")
g1.plot ( )
paws ( )
# The following is good but comment it out if you want to save time.
for i in range (8) :
   g1.quick_plot (titles = ["Mesh follows velocity field one time step",
         "(Animation)"], phi = 5. + 10. * (i + 1))
   paws ()
for i in range (4) :
   g1.quick_plot (theta = 45. - 10. * (i + 1))
   paws ( )



#
# Author: Travis Oliphant 2001
#

from filter_design import tf2zpk, zpk2tf, normalize
from Numeric import product, zeros, concatenate, \
     array, dot, transpose, arange, ones, Float
import Numeric
import scipy.interpolate as interpolate
import scipy.integrate as integrate
import scipy.linalg as linalg
from scipy_base import r_, c_, eye, real, atleast_1d, atleast_2d, poly, \
     squeeze, diag, asarray
from Matrix import Matrix as Mat

def tf2ss(num, den):
    """Transfer function to state-space representation.

    Inputs:

      num, den -- sequences representing the numerator and denominator polynomials.

    Outputs:

      A, B, C, D -- state space representation of the system.
    """
    # Controller canonical state-space representation.
    #  if M+1 = len(num) and K+1 = len(den) then we must have M <= K 
    #  states are found by asserting that X(s) = U(s) / D(s)
    #  then Y(s) = N(s) * X(s) 
    #
    #   A, B, C, and D follow quite naturally.
    #
    num, den = normalize(num, den)   # Strips zeros, checks arrays
    nn = len(num.shape)
    if nn == 1:
        num = asarray([num],num.typecode())
    M = num.shape[1] 
    K = len(den) 
    if (M > K):
        raise ValueError, "Improper transfer function."
    if (M == 0 or K == 0):  # Null system
        return array([],Float), array([], Float), array([], Float), \
               array([], Float)

    # pad numerator to have same number of columns has denominator
    num = c_[zeros((num.shape[0],K-M),num.typecode()), num]

    if num.shape[-1] > 0:
        D = num[:,0]
    else:
        D = array([],Float)

    if K == 1:
        return array([], Float), array([], Float), array([], Float), D
    
    frow = -array([den[1:]])
    A = r_[frow, eye(K-2, K-1)]
    B = eye(K-1, 1)
    C = num[:,1:] - num[:,0] * den[1:]
    return A, B, C, D

def none_to_empty(arg):
    if arg is None:
        return []
    else:
        return arg

def abcd_normalize(A=None, B=None, C=None, D=None):
    """Check state-space matrices and ensure they are rank-2.
    """
    A, B, C, D = map(none_to_empty, (A, B, C, D))
    A, B, C, D = map(atleast_2d, (A, B, C, D))

    if ((len(A.shape) > 2) or (len(B.shape) > 2) or \
        (len(C.shape) > 2) or (len(D.shape) > 2)):
        raise ValueError, "A, B, C, D arrays can be no larger than rank-2."

    MA, NA = A.shape
    MB, NB = B.shape
    MC, NC = C.shape
    MD, ND = D.shape

    if (MC == 0) and (NC == 0) and (MD != 0) and (NA != 0):
        MC, NC = MD, NA
        C = zeros((MC, NC))
    if (MB == 0) and (NB == 0) and (MA != 0) and (ND != 0):
        MB, NB = MA, ND
        B = zeros(MB, NB)
    if (MD == 0) and (ND == 0) and (MC != 0) and (NB != 0):
        MD, ND = MC, NB
        D = zeros(MD, ND)
    if (MA == 0) and (NA == 0) and (MB != 0) and (NC != 0):
        MA, NA = MB, NC
        A = zeros(MA, NA)

    if MA != NA:
        raise ValueError, "A must be square."
    if MA != MB:
        raise ValueError, "A and B must have the same number of rows."
    if NA != NC:
        raise ValueError, "A and C must have the same number of columns."
    if MD != MC:
        raise ValueError, "C and D must have the same number of rows."
    if ND != NB:
        raise ValueErrro, "B and D must have the same number of columns."

    return A, B, C, D
    
def ss2tf(A, B, C, D, input=0):
    """State-space to transfer function.

    Inputs:

      A, B, C, D -- state-space representation of linear system.
      input -- For multiple-input systems, the input to use.

    Outputs:

      num, den -- Numerator and denominator polynomials (as sequences)
                  respectively.
    """
    # transfer function is C (sI - A)**(-1) B + D
    A, B, C, D = map(asarray, (A, B, C, D))
    # Check consistency and 
    #     make them all rank-2 arrays
    A, B, C, D = abcd_normalize(A, B, C, D)

    nout, nin = D.shape
    if input >= nin:
        raise ValueError, "System does not have the input specified."

    # make MOSI from possibly MOMI system.
    if B.shape[-1] != 0:
        B = B[:,input]
    B.shape = (B.shape[0],1)
    if D.shape[-1] != 0:
        D = D[:,input]

    den = poly(A)

    if (product(B.shape) == 0) and (product(C.shape) == 0):
        num = Numeric.ravel(D)
        if (product(D.shape) == 0) and (product(A.shape) == 0):
            den = []
        end
        return num, den

    num_states = A.shape[0] 
    type_test = A[:,0] + B[:,0] + C[0,:] + D
    num = Numeric.zeros((nout, num_states+1),type_test.typecode())
    for k in range(nout):
        Ck = atleast_2d(C[k,:])
        num[k] = poly(A - dot(B,Ck)) + (D[k]-1)*den

    return num, den

def zpk2ss(z,p,k):
    """Zero-pole-gain representation to state-space representation

    Inputs:

      z, p, k -- zeros, poles (sequences), and gain of system

    Outputs:

      A, B, C, D -- state-space matrices.
    """
    return tf2ss(*zpk2tf(z,p,k))

def ss2zpk(A,B,C,D,input=0):
    """State-space representation to zero-pole-gain representation.

    Inputs:

      A, B, C, D -- state-space matrices.
      input -- for multiple-input systems, the input to use.

    Outputs:

      z, p, k -- zeros and poles in sequences and gain constant.
    """    
    return tf2zpk(*ss2tf(A,B,C,D,input=input))

class lti:
    """Linear Time Invariant class which simplifies representation.
    """
    def __init__(self,*args,**kwords):
        """Initialize the LTI system using either:
           (numerator, denominator)
           (zeros, poles, gain)
           (A, B, C, D) -- state-space.
        """
        N = len(args)
        if N == 2:  # Numerator denominator transfer function input
            self.__dict__['num'], self.__dict__['den'] = normalize(*args)
            self.__dict__['zeros'], self.__dict__['poles'], \
            self.__dict__['gain'] = tf2zpk(*args)
            self.__dict__['A'], self.__dict__['B'], \
                                self.__dict__['C'], \
                                self.__dict__['D'] = tf2ss(*args)
            self.inputs = 1
            if len(self.num.shape) > 1:
                self.outputs = self.num.shape[0]
            else:
                self.outputs = 1
        elif N == 3:      # Zero-pole-gain form
            self.__dict__['zeros'], self.__dict__['poles'], \
                                    self.__dict__['gain'] = args
            self.__dict__['num'], self.__dict__['den'] = zpk2tf(*args)
            self.__dict__['A'], self.__dict__['B'], \
                                self.__dict__['C'], \
                                self.__dict__['D'] = zpk2ss(*args)
            self.inputs = 1
            if len(self.zeros.shape) > 1:
                self.outputs = self.zeros.shape[0]
            else:
                self.outputs = 1
        elif N == 4:       # State-space form
            self.__dict__['A'], self.__dict__['B'], \
                                self.__dict__['C'], \
                                self.__dict__['D'] = abcd_normalize(*args)
            self.__dict__['zeros'], self.__dict__['poles'], \
                                    self.__dict__['gain'] = ss2zpk(*args)
            self.__dict__['num'], self.__dict__['den'] = ss2tf(*args)
            self.inputs = B.shape[-1]
            self.outputs = C.shape[0]
        else:
            raise ValueError, "Needs 2, 3, or 4 arguments."

    def __setattr__(self, attr, val):
        if attr in ['num','den']:
            self.__dict__[attr] = val
            self.__dict__['zeros'], self.__dict__['poles'], \
                                    self.__dict__['gain'] = \
                                    tf2zpk(self.num, self.den)
            self.__dict__['A'], self.__dict__['B'], \
                                self.__dict__['C'], \
                                self.__dict__['D'] = \
                                tf2ss(self.num, self.den)
        elif attr in ['zeros', 'poles', 'gain']:
            self.__dict__[attr] = val
            self.__dict__['num'], self.__dict__['den'] = \
                                  zpk2tf(self.zeros,
                                         self.poles, self.gain)
            self.__dict__['A'], self.__dict__['B'], \
                                self.__dict__['C'], \
                                self.__dict__['D'] = \
                                zpk2ss(self.zeros,
                                       self.poles, self.gain)
        elif attr in ['A', 'B', 'C', 'D']:
            self.__dict__[attr] = val
            self.__dict__['zeros'], self.__dict__['poles'], \
                                    self.__dict__['gain'] = \
                                    ss2zpk(self.A, self.B,
                                           self.C, self.D)
            self.__dict__['num'], self.__dict__['den'] = \
                                  ss2tf(self.A, self.B,
                                        self.C, self.D)            
        else:
            self.__dict__[attr] = val

    def impulse(self, X0=None, T=None, N=None):
        return impulse(self, X0=X0, T=T, N=N)

    def step(self, X0=None, T=None, N=None):
        return step(self, X0=X0, T=T, N=N)

    def output(self, U, T, X0=None):
        return lsim(self, U, T, X0=X0)


def lsim2(system, U, T, X0=None):
    """Simulate output of a continuous-time linear system, using ODE solver.

    Inputs:

      system -- an instance of the LTI class or a tuple describing the
                system.  The following gives the number of elements in
                the tuple and the interpretation.
                  2 (num, den)
                  3 (zeros, poles, gain)
                  4 (A, B, C, D)
      U -- an input array describing the input at each time T
           (linear interpolation is assumed between given times).
           If there are multiple inputs, then each column of the 
           rank-2 array represents an input.
      T -- the time steps at which the input is defined and at which
           the output is desired.
      X0 -- (optional, default=0) the initial conditions on the state vector.

    Outputs: (T, yout, xout)

      T -- the time values for the output.
      yout -- the response of the system.
      xout -- the time-evolution of the state-vector.
    """
    # system is an lti system or a sequence
    #  with 2 (num, den)
    #       3 (zeros, poles, gain)
    #       4 (A, B, C, D)
    #  describing the system
    #  U is an input vector at times T
    #   if system describes multiple outputs
    #   then U can be a rank-2 array with the number of columns
    #   being the number of inputs

    # rather than use lsim, use direct integration and matrix-exponential.
    if isinstance(system, lti):        
        sys = system
    else:
        sys = lti(*system)
    U = atleast_1d(U)
    T = atleast_1d(T)
    if len(U.shape) == 1:
        U.shape = (U.shape[0],1)
    sU = U.shape        
    if len(T.shape) != 1:
        raise ValueError, "T must be a rank-1 array."
    if sU[0] != len(T):
        raise ValueError, "U must have the same number of rows as elements in T."
    if sU[1] != sys.inputs:
        raise ValueError, "System does not define that many inputs."

    if X0 is None:
        X0 = zeros(sys.B.shape[0],sys.A.typecode())

    # for each output point directly integrate assume zero-order hold
    #   or linear interpolation.

    ufunc = interpolate.linear_1d(T, U, axis=0, bounds_error=0, fill_value=0)

    def fprime(x, t, sys, ufunc):
        return dot(sys.A,x) + squeeze(dot(sys.B,ufunc([t])))

    xout = integrate.odeint(fprime, X0, T, args=(sys, ufunc))
    yout = dot(sys.C,transpose(xout)) + dot(sys.D,transpose(U))
    return T, squeeze(transpose(yout)), xout


def lsim(system, U, T, X0=None, interp=1):
    """Simulate output of a continuous-time linear system.

    Inputs:

      system -- an instance of the LTI class or a tuple describing the
                system.  The following gives the number of elements in
                the tuple and the interpretation.
                  2 (num, den)
                  3 (zeros, poles, gain)
                  4 (A, B, C, D)
      U -- an input array describing the input at each time T
           (interpolation is assumed between given times).
           If there are multiple inputs, then each column of the 
           rank-2 array represents an input.
      T -- the time steps at which the input is defined and at which
           the output is desired.
      X0 -- (optional, default=0) the initial conditions on the state vector.
      interp -- linear (1) or zero-order hold (0) interpolation

    Outputs: (T, yout, xout)

      T -- the time values for the output.
      yout -- the response of the system.
      xout -- the time-evolution of the state-vector.
    """
    # system is an lti system or a sequence
    #  with 2 (num, den)
    #       3 (zeros, poles, gain)
    #       4 (A, B, C, D)
    #  describing the system
    #  U is an input vector at times T
    #   if system describes multiple inputs
    #   then U can be a rank-2 array with the number of columns
    #   being the number of inputs
    if isinstance(system, lti):
        sys = system
    else:
        sys = lti(*system)
    U = atleast_1d(U)
    T = atleast_1d(T)
    if len(U.shape) == 1:
        U.shape = (U.shape[0],1)
    sU = U.shape        
    if len(T.shape) != 1:
        raise ValueError, "T must be a rank-1 array."
    if sU[0] != len(T):
        raise ValueError, "U must have the same number of rows as elements in T."
    if sU[1] != sys.inputs:
        raise ValueError, "System does not define that many inputs."

    if X0 is None:
        X0 = zeros(sys.B.shape[0],sys.A.typecode())

    xout = zeros((len(T),sys.B.shape[0]),sys.A.typecode())
    xout[0] = X0
    A = sys.A
    AT, BT = transpose(sys.A), transpose(sys.B)
    dt = T[1]-T[0]
    lam, v = linalg.eig(A)
    vt = transpose(v)
    vti = linalg.inv(vt)
    GT = dot(dot(vti,diag(Numeric.exp(dt*lam))),vt).astype(xout.typecode())
    ATm1 = linalg.inv(AT)
    ATm2 = dot(ATm1,ATm1)
    I = eye(A.shape[0],typecode=A.typecode())
    GTmI = GT-I
    F1T = dot(dot(BT,GTmI),ATm1)
    if interp:
        F2T = dot(BT,dot(GTmI,ATm2)/dt - ATm1)

    for k in xrange(1,len(T)):
        dt1 = T[k] - T[k-1]
        if dt1 != dt:
            dt = dt1
            GT = dot(dot(vti,diag(Numeric.exp(dt*lam))),vt).astype(xout.typecode())
            GTmI = GT-I
            F1T = dot(dot(BT,GTmI),ATm1)
            if interp:
                F2T = dot(BT,dot(GTmI,ATm2)/dt - ATm1)

        xout[k] = dot(xout[k-1],GT) + dot(U[k-1],F1T)
        if interp:
            xout[k] = xout[k] + dot((U[k]-U[k-1]),F2T)

    yout = squeeze(dot(U,transpose(sys.D))) + squeeze(dot(xout,transpose(sys.C)))
    return T, squeeze(yout), squeeze(xout)


def impulse(system, X0=None, T=None, N=None):
    """Impulse response of continuous-time system.

    Inputs:

      system -- an instance of the LTI class or a tuple with 2, 3, or 4
                elements representing (num, den), (zero, pole, gain), or
                (A, B, C, D) representation of the system.
      X0 -- (optional, default = 0) inital state-vector.
      T -- (optional) time points (autocomputed if not given).
      N -- (optional) number of time points to autocompute (100 if not given).

    Ouptuts: (T, yout)

      T -- output time points,
      yout -- impulse response of system (except possible singularities at 0).
    """
    if isinstance(system, lti):
        sys = system
    else:
        sys = lti(*system)
    if X0 is None:
        B = sys.B
    else:
        B = sys.B + X0
    if N is None:
        N = 100
    if T is None:
        vals = linalg.eigvals(sys.A)
        tc = 1.0/min(abs(real(vals)))
        T = arange(0,7*tc,7*tc / float(N))
    h = zeros(T.shape, sys.A.typecode())
    s,v = linalg.eig(sys.A)
    vi = linalg.inv(v)
    C = sys.C
    for k in range(len(h)):
        es = diag(Numeric.exp(s*T[k]))
        eA = (dot(dot(v,es),vi)).astype(h.typecode())
        h[k] = squeeze(dot(dot(C,eA),B))
    return T, h

def step(system, X0=None, T=None, N=None):
    """Step response of continuous-time system.

    Inputs:

      system -- an instance of the LTI class or a tuple with 2, 3, or 4
                elements representing (num, den), (zero, pole, gain), or
                (A, B, C, D) representation of the system.
      X0 -- (optional, default = 0) inital state-vector.
      T -- (optional) time points (autocomputed if not given).
      N -- (optional) number of time points to autocompute (100 if not given).

    Ouptuts: (T, yout)

      T -- output time points,
      yout -- step response of system.
    """
    if isinstance(system, lti):
        sys = system
    else:
        sys = lti(*system)
    if N is None:
        N = 100
    if T is None:
        vals = linalg.eigvals(sys.A)
        tc = 1.0/min(abs(real(vals)))
        T = arange(0,7*tc,7*tc / float(N))
    U = ones(T.shape, sys.A.typecode())
    vals = lsim(sys, U, T, X0=X0)
    return vals[0], vals[1]
    



"""
Optimization Tools
==================

 A collection of general-purpose optimization routines.

   fmin        --  Nelder-Mead Simplex algorithm
                     (uses only function calls)
   fmin_powell --  Powell's (modified) level set method (uses only 
                     function calls)
   fmin_cg     --  Non-linear (Polak-Rubiere) conjugate gradient algorithm
                     (can use function and gradient).
                     
   fmin_bfgs   --  Quasi-Newton method (can use function and gradient)
   fmin_ncg    --  Line-search Newton Conjugate Gradient (can use
                     function, gradient and hessian).
   leastsq     --  Minimize the sum of squares of M equations in
                     N unknowns given a starting estimate.

  Constrained Optimizers (multivariate)

   fmin_l_bfgs_b -- Zhu, Byrd, and Nocedal's L-BFGS-B constrained optimizer
                      (if you use this please quote their papers -- see help)

   fmin_tnc      -- Truncated Newton Code originally written by Stephen Nash and
                      adapted to C by Jean-Sebastien Roy.

   fmin_cobyla   -- Contrained Optimization BY Linear Approximation

  Global Optimizers

   anneal      --  Simulated Annealing
   brute       --  Brute Force searching Optimizer

  Scalar function minimizers
 
   fminbound   --  Bounded minimization of a scalar function.
   brent       --  1-D function minimization using Brent method.
   golden      --  1-D function minimization using Golden Section method
   bracket     --  Bracket a minimum (given two starting points)
  
 Also a collection of general_purpose root-finding routines.

   fsolve      --  Non-linear multi-variable equation solver.

  Scalar function solvers

   brentq      --  quadratic interpolation Brent method
   brenth      --  Brent method (modified by Harris with
                     hyperbolic extrapolation)
   ridder      --  Ridder's method
   bisect      --  Bisection method
   newton      --  Secant method or Newton's method

   fixed_point -- Single-variable fixed-point solver.

 Utility Functions

   line_search -- Return a step that satisfies the strong Wolfe conditions.
   check_grad  -- Check the supplied derivative using finite difference
                   techniques.
"""

postpone_import = 1



#
# Machine arithmetics - determine the parameters of the
# floating-point arithmetic system
#
# Author: Pearu Peterson, September 2003
#

__all__ = ['MachAr','machar_double','machar_single']

from Numeric import array

class MachAr:
    """Diagnosing machine parameters.

    The following attributes are available:

    ibeta  - radix in which numbers are represented
    it     - number of base-ibeta digits in the floating point mantissa M
    machep - exponent of the smallest (most negative) power of ibeta that,
             added to 1.0,
             gives something different from 1.0
    eps    - floating-point number beta**machep (floating point precision)
    negep  - exponent of the smallest power of ibeta that, substracted
             from 1.0, gives something different from 1.0
    epsneg - floating-point number beta**negep
    iexp   - number of bits in the exponent (including its sign and bias)
    minexp - smallest (most negative) power of ibeta consistent with there
             being no leading zeros in the mantissa
    xmin   - floating point number beta**minexp (the smallest (in
             magnitude) usable floating value)
    maxexp - smallest (positive) power of ibeta that causes overflow
    xmax   - (1-epsneg)* beta**maxexp (the largest (in magnitude)
             usable floating value)
    irnd   - in range(6), information on what kind of rounding is done
             in addition, and on how underflow is handled
    ngrd   - number of 'guard digits' used when truncating the product
             of two mantissas to fit the representation

    epsilon - same as eps
    tiny    - same as xmin
    huge    - same as xmax
    precision   - int(-log10(eps))
    resolution  - 10**(-precision)

    Reference:
      Numerical Recipies.
    """
    def __init__(self,
                 float_conv=float,
                 int_conv=int,
                 float_to_float=float,
                 float_to_str = lambda v:'%24.16e' % v,
                 title = 'Python floating point number',
                 ):
        """
          float_conv - convert integer to float (array)
          int_conv   - convert float (array) to integer
          float_to_float - convert float array to float
          float_to_str - convert array float to str
          title        - description of used floating point numbers
        """
        one = float_conv(1)
        two = one + one
        zero = one - one

        # Determine ibeta and beta
        a = one
        while 1:
            a = a + a
            temp = a + one
            temp1 = temp - a
            if temp1 - one != zero:
                break
        b = one
        while 1:
            b = b + b
            temp = a + b
            itemp = int_conv(temp-a)
            if itemp != 0:
                break
        ibeta = itemp
        beta = float_conv(ibeta)

        # Determine it and irnd
        it = 0
        b = one
        while 1:
            it = it + 1
            b = b * beta
            temp = b + one
            temp1 = temp - b
            if temp1 - one != zero:
                break

        betah = beta / two
        a = one
        while 1:
            a = a + a
            temp = a + one
            temp1 = temp - a
            if temp1 - one != zero:
                break
        temp = a + betah
        irnd = 0
        if temp-a != zero:
            irnd = 1
        tempa = a + beta
        temp = tempa + betah
        if irnd==0 and temp-tempa != zero:
            irnd = 2

        # Determine negep and epsneg
        negep = it + 3
        betain = one / beta
        a = one
        for i in range(negep):
            a = a * betain
        b = a
        while 1:
            temp = one - a
            if temp-one != zero:
                break
            a = a * beta
            negep = negep - 1
        negep = -negep
        epsneg = a

        # Determine machep and eps
        machep = - it - 3
        a = b

        while 1:
            temp = one + a
            if temp-one != zero:
                break
            a = a * beta
            machep = machep + 1
        eps = a

        # Determine ngrd
        ngrd = 0
        temp = one + eps
        if irnd==0 and temp*one - one != zero:
            ngrd = 1

        # Determine iexp
        i = 0
        k = 1
        z = betain
        t = one + eps
        nxres = 0
        while 1:
            y = z
            z = y*y
            a = z*one # Check here for underflow
            temp = z*t
            if a+a == zero or abs(z)>=y:
                break
            temp1 = temp * betain
            if temp1*beta == z:
                break
            i = i + 1
            k = k + k
        if ibeta != 10:
            iexp = i + 1
            mx = k + k
        else:
            iexp = 2
            iz = ibeta
            while k >= iz:
                iz = iz * ibeta
                iexp = iexp + 1
            mx = iz + iz - 1

        # Determine minexp and xmin
        while 1:
            xmin = y
            y = y * betain
            a = y * one
            temp = y * t
            if a+a != zero and abs(y) < xmin:
                k = k + 1
                temp1 = temp * betain
                if temp1*beta == y and temp != y:
                    nxres = 3
                    xmin = y
                    break
            else:
                break
        minexp = -k

        # Determine maxexp, xmax
        if mx <= k + k - 3 and ibeta != 10:
            mx = mx + mx
            iexp = iexp + 1
        maxexp = mx + minexp
        irnd = irnd + nxres
        if irnd >= 2:
            maxexp = maxexp - 2
        i = maxexp + minexp
        if ibeta == 2 and not i:
            maxexp = maxexp - 1
        if i > 20:
            maxexp = maxexp - 1
        if a != y:
            maxexp = maxexp - 2
        xmax = one - epsneg
        if xmax*one != xmax:
            xmax = one - beta*epsneg
        xmax = xmax / (xmin*beta*beta*beta)
        i = maxexp + minexp + 3
        for j in range(i):
            if ibeta==2:
                xmax = xmax + xmax
            else:
                xmax = xmax * beta

        self.ibeta = ibeta
        self.it = it
        self.negep = negep
        self.epsneg = float_to_float(epsneg)
        self._str_epsneg = float_to_str(epsneg)
        self.machep = machep
        self.eps = float_to_float(eps)
        self._str_eps = float_to_str(eps)
        self.ngrd = ngrd
        self.iexp = iexp
        self.minexp = minexp
        self.xmin = float_to_float(xmin)
        self._str_xmin = float_to_str(xmin)
        self.maxexp = maxexp
        self.xmax = float_to_float(xmax)
        self._str_xmax = float_to_str(xmax)
        self.irnd = irnd

        self.title = title
        # Commonly used parameters
        self.epsilon = self.eps
        self.tiny = self.xmin
        self.huge = self.xmax

        import math
        self.precision = int(-math.log10(float_to_float(self.eps)))
        ten = two + two + two + two + two
        resolution = ten ** (-self.precision)
        self.resolution = float_to_float(resolution)

    def __str__(self):
        return '''\
Machine parameters for %(title)s
---------------------------------------------------------------------
ibeta=%(ibeta)s it=%(it)s iexp=%(iexp)s ngrd=%(ngrd)s irnd=%(irnd)s
machep=%(machep)s     eps=%(_str_eps)s (beta**machep == epsilon)
negep =%(negep)s  epsneg=%(_str_epsneg)s (beta**epsneg)
minexp=%(minexp)s   xmin=%(_str_xmin)s (beta**minexp == tiny)
maxexp=%(maxexp)s    xmax=%(_str_xmax)s ((1-epsneg)*beta**maxexp == huge)
---------------------------------------------------------------------
''' % self.__dict__

machar_double = MachAr(lambda v:array([v],'d'),
                       lambda v:v.astype('i')[0],
                       lambda v:array(v[0],'d'),
                       lambda v:'%24.16e' % array(v[0],'d'),
                       'Numeric double precision floating point number')

machar_single = MachAr(lambda v:array([v],'f'),
                       lambda v:v.astype('i')[0],
                       lambda v:array(v[0],'f'),  #
                       lambda v:'%15.7e' % array(v[0],'f'),
                       'Numeric single precision floating point number')

if __name__ == '__main__':
    print MachAr()
    print machar_double
    print machar_single



# Copyright (c) 1996, 1997, The Regents of the University of California.
# All rights reserved.  See Legal.htm for full text and disclaimer.

GraphicsError = "GraphicsError"
from types import *
from graftypes import *
import os
try:
   graphics = os.environ["PYGRAPH"]
except KeyError:
   print "env variable PYGRAPH is not set, so assuming Gist graphics."
   print "...this can be overruled by the 'graphics' keyword argument"
   print "   to Graph2d or Graph3d."
   graphics = "Gist"

 
if graphics [0:3] == "Nar" :
   import NarPlotter
   Graphics = NarPlotter
elif graphics == "Gist" :
   import GistPlotter
   Graphics = GistPlotter
else :
   raise GraphicsError , \
      graphics + " is an unknown graphics package. Check PYGRAPH " + \
         "environment variable."
 
# The following is so I know about arrays:
from Numeric import *
from scipy_base.fastumath import *
from shapetest import *

class Graph :
 
   """
   g = Graph ( <keyword arguments> ) abstracts all the information
   that is common to 2d, 3d, and 4d graphs, and will be inherited from
   by all of those types. It is never intended to be instantiated
   except via a derived class. The keyword arguments are:
 
       plotter = <Plotter object> or a sequence of <Plotter object>s
              if you have a plotter object or a sequence of them that
              you want the graph to use when plotting itself. I
              recommend against this; a graph can create its own
              plotter if you don't give it one.
       filename = <string> or a sequence of <string>s if you want to
              connect to a Plotter with the named file(s). (The default
              is " ".) Only one of the keywords 'plotter' or 'filename'
              is allowed, and both are optional.
              NOTE: the possibility of a sequence of file names or
              plotters allows one to display Plotter on one's
              own machine as well as one or more remote machines.
              In Gist, the filename argument is the same as the
              display argument (below).
       hardcopy = <string> Allows the user to specify a hardcopy file
              for this Plotter object. If string ends in ".ps", this will
              be postscript; it ".cgm", then it will be cgm.
              This is valid only for Gist; ignored for Narcisse.
       display = <string> or a sequence of <string>s if you want to
              display on the named hosts. The form of <string> is
              the usual "hostname:server.screen". The purpose of
              this argument is to allow you to continue without
              exiting python if you have to open a Gist window
              without the DISPLAY environment variable having been
              set, or if you want to open Gist windows on more
              than one host.
       graphics = <string> or a sequence of <string>s if you want to
              specify which graphics the particular plotter connects to.
              Currently the values allowed are "Nar" and "Gist".
              This argument is meaningless if you supply a list
              of plotters. If it is a scalar and you supply a list
              of filenames, all plotters opened will be that type.
              If it is a vector, then it must match the list of
              filenames in size and correspond to the filename.
       titles = <value> where <value> is a string or a sequence
                of up to four strings, giving the titles in the
                order bottom, top, left, right.
       title_colors = <value> where value is an integer or string
                or a sequence of up to four integers or strings giving
                the colors of the titles.
       axis_labels = <value> where <value> is a string or
                for a 2d Graph: a sequence of up to three strings
                representing the labels of the x axis, the y axis,
                and the right y axis. for a 3d Graph: a sequence of
                up to four strings representing the labels of the
                x axis, the y axis, the z axis, and the right y axis.
       axis_limits = <value> where <value> is a pair [xmin, xmax] or
                 for a 2d Graph: a sequence of up to three pairs,
                 where the second would be the y limits and the third
                 the right y limits. for a 3d or 4d graph:
                 a sequence of up to five pairs, where the second
                 would be the y limits, the third the z limits,
                 the fourth the c limits, and the fifth the right
                 y limits. axis_limits = "defaults" will let the
                 graphics calculate the limits.
          x_axis_limits, y_axis_limits, yr_axis_limits, z_axis_limits,
                 and c_axis_limits are used to change the limits on
                 individual axes. They may be set to a pair or to
                 "defaults".
       axis_scales = "linlin", "linlog", "loglin", or "loglog" for
                2d graphs with no separate right y axis; if there is
                a separate yr axis, and it is not to default to
                linear, then the scales must be expressed as a
                sequence of three values "log" or "lin" with the
                third one (for yr) "log". For 3d and 4d plots,
                one or a sequence of up to five values "log" or "lin"
                which are specified in the order x, y, z, c, and yr.
          x_axis_scale, y_axis_scale, yr_axis_scale, z_axis_scale,
                 and c_axis_scale are used to change the scale on
                 individual axes. They may be set to "log" or "lin".
       text = <value> where value is one or a sequence of strings
                representing texts to be placed on the plot.
       text_color = <value> where <value> is one or a sequence
                of color numbers or names giving colors for the
                texts.
       text_size = <value> where <value> is one or a sequence of
                integers giving (roughly) the number of characters
                in a line on the graph.
       text_pos = <value> where <value> is a pair or a sequence or
                reals between 0. and 1.0 giving the relative
                position of the lower left corner of a text
                in the graphics window.
       xyequal = 0/1 If 1, the axis limits will be adjusted so that
                both axes are to the same scale.
       sync = 0 or 1 (1 to synchronize before plotting)
                defaults to 1, otherwise plots may get garbled.
       color_bar = 0 or 1 (1 enables plotting of a color bar on
                any graphs for which it is meaningful (colored contour
                plots, filled contour plots, cell arrays, filled
                meshes and polygons).
       color_bar_pos (ignored unless a color bar is actually plotted)
                is a 2d array [ [xmin, ymin], [xmax, ymax]] specifying
                where (in window coordinates) the diagonally opposite
                corners of the color bar are to be placed.
   """

   def _initialize_generics ( self , keywords ) :
      """_initialize_generics ( keywords ) is called from __init__ to
      set all those things which are generic to a graph: sync, 
      axis_labels, axis_scales, axis_limits, etc., and
      titles, title_colors, text, text_color, text_size, and text_pos.
      Gives no default if keyword is not present, so can be called
      by change.
      Formerly, this routine used to do color conversions between
      graphics versions. Not any more; that has been relegated to
      the Plotter level.
      This routine has also been changed to delete generic items from
      the keywords so that other items can be changed as well. It was
      not intuitive why calling 'change' would not change any item
      you pleased.
      """
      if keywords.has_key ("sync") :
         self._sync = keywords ["sync"]
         del keywords ["sync"]
      if keywords.has_key ("titles") :
         self._titles = keywords ["titles"]
         if type (self._titles) == StringType :
            self._titles = [self._titles]
         del keywords ["titles"]
      if keywords.has_key ("title_colors") :
         self._title_colors = keywords ["title_colors"]
         del keywords ["title_colors"]
      if keywords.has_key ("text") :
         self._text = keywords ["text"]
         del keywords ["text"]
      if keywords.has_key ("text_color") :
         self._text_color = keywords ["text_color"]
         del keywords ["text_color"]
      if keywords.has_key ("text_size") :
         self._text_size = keywords ["text_size"]
         del keywords ["text_size"]
      if keywords.has_key ("text_pos") :
         self._text_pos = keywords ["text_pos"]
         del keywords ["text_pos"]
      if keywords.has_key ("tosys") :
         self._tosys = keywords ["tosys"]
         del keywords ["tosys"]
      self._init_axis_labels (keywords)
      # handle keywords which take care of all axes
      if keywords.has_key ("axis_scales") :
         axs = keywords ["axis_scales"]
         if is_scalar (axs) and self._scale_dict_2d.has_key (axs) :
            axs = self._scale_dict_2d [axs]
         if is_scalar (axs) :
            self._axis_scales [0] = axs
         else :
            for i in range (len (axs)) :
               self._axis_scales [i] = axs [i]
         del keywords ["axis_scales"]
      if keywords.has_key ("axis_limits") :
         axl = keywords ["axis_limits"]
         if axl == "defaults" :
            for i in range (self._no_of_axes) :
                self._axis_limits [i] = [0., 0.]
         elif is_scalar (axl) :
            raise self._AxisSpecError , \
               "Axis limits must be a point."
         elif type (axl) == ListType or type (axl) == ArrayType :
            if type (axl [0]) != ListType and type (axl [0]) != ArrayType :
               self._axis_limits [0] = axl
            else :
               for i in range (len (axl)) :
                  self._axis_limits [i] = axl [i]
         else :
            raise   self._AxisSpecError , \
               "Axis limits are incomprehensible."
         del keywords ["axis_limits"]
      if keywords.has_key ("color_bar") :
         self._color_bar = keywords ["color_bar"]
      else :
         self._color_bar = 0
      if keywords.has_key ("color_bar_pos") :
         self._color_bar_pos = keywords ["color_bar_pos"]
      else :
         self._color_bar_pos = None
      # handle individual axis keywords
      for i in range (self._no_of_axes) :
         if keywords.has_key (self._scale_keywords [i]) :
            self._axis_scales [i] = keywords [self._scale_keywords [i]]
            del keywords [self._scale_keywords [i]]
         if keywords.has_key (self._limits_keywords [i]) :
            self._axis_limits [i] = keywords [self._limits_keywords [i]]
            del keywords [self._limits_keywords [i]]
      if keywords.has_key ( "xyequal") :
         self._xyequal = keywords ["xyequal"]
         del keywords ["xyequal"]

   # common to all Graph classes
   _GraphInitError = "GraphInitError"
   _GraphSpecError = "GraphSpecError"
   _AxisSpecError = "AxisSpecError"

   _scale_dict_2d = { "linlin" : ["lin", "lin"] ,
                      "linlog" : ["lin", "log"] ,
                      "loglin" : ["log", "lin"] ,
                      "loglog" : ["log", "log"] }

   def __init__ ( self , * kwds , ** keywords ) :
      global Graphics
      self._send_coordinates = 1
      if len (kwds) == 1 :
         keywords = kwds[0]
      if not hasattr (self, "_filename_list") :
         self._filename_list = []
      if not hasattr (self, "_plotter_list") :
         self._plotter_list = []
      if ( keywords.has_key ("filename") or keywords.has_key ("display")) \
         and keywords.has_key ("plotter") :
         raise self._GraphInitError , \
            "Only one keyword, 'filename (display)' or 'plotter' allowed."
      if keywords.has_key ("hardcopy") :
         self._hardcopy = keywords ["hardcopy"]
      else :
         self._hardcopy = ""
      self._graphics_list = [Graphics]
      if keywords.has_key ("filename") or keywords.has_key ("display") :
         if keywords.has_key ("graphics") :
            if keywords ["graphics"] == "" or keywords ["graphics"] is None :
                self._graphics_list = [Graphics]
            elif keywords ["graphics"] == "Nar" or \
                 keywords ["graphics"] == "Narcisse" :
                self._graphics_list = [NarPlotter]
            elif keywords ["graphics"] == "Gist" :
                self._graphics_list = [GistPlotter]
            else :
                self._graphics_list = keywords ["graphics"]
            if is_scalar (self._graphics_list) :
               self._graphics_list = [self._graphics_list]
            for i in range (len (self._graphics_list)) :
               if self._graphics_list [i] == "Gist" :
                  self._graphics_list [i] = GistPlotter
               elif self._graphics_list [i] == "Nar" or \
                    self._graphics_list [i] == "Narcisse" :
                  self._graphics_list [i] = NarPlotter
               else :
                  raise self._GraphInitError , "Illegal graphics option: " + \
                      self._graphics_list [i]
         if keywords.has_key ("filename") :
            fn = keywords ["filename"]
         else :
            fn = keywords ["display"]
         if is_scalar (fn) and not fn in self._filename_list :
            self._filename_list.append (fn)
            self._plotter_list.append (-1)
         else :
            for i in range (len (fn)) :
               if not fn [i] in self._filename_list :
                  self._filename_list.append (fn [i])
                  self._plotter_list.append (-1)
         self._plotters_started = 0
      elif keywords.has_key ("plotter") :
         pl = keywords ["plotter"]
         if is_scalar (pl) and not pl in self._plotter_list :
            self._plotter_list.append (pl)
            self._filename_list.append (" ")
         else :
            for i in range (len (pl)) :
               if not pl [i] in self._plotter_list :
                  self._plotter_list.append (pl [i])
                  self._filename_list.append (" ")
         self._plotters_started = 1
      self._titles = [" ", " ", " ", " "]
      self._title_colors = "fg"
      self._text = " "
      self._text_color = 1
      self._text_size = 0
      self._text_pos = [0.,0.]
      self._tosys = 0
      self._sync = 1
      self._xyequal = 0
      self._initialize_generics ( keywords )

   def _init_axis_labels ( self , keywords ) :
      """_init_axis_labels ( kw ) sets the axis labels if keywords are present.
      This routine can be called by quick_plot in a dervied class.
      """
      if keywords.has_key ("axis_labels") :
         axl = keywords ["axis_labels"]
         if is_scalar (axl) :
            self._axis_labels [0] = axl
         else :
            for i in range (len (axl)) :
               self._axis_labels [i] = axl [i]
         del keywords ["axis_labels"]
      if keywords.has_key ("gnomon") :
         self._gnomon = keywords ["gnomon"]
         del keywords ["gnomon"]
      else :
         self._gnomon = 0
      for i in range (self._no_of_axes) :
         if keywords.has_key (self._label_keywords [i]) :
            self._axis_labels [i] = keywords [self._label_keywords [i]]
            del keywords [self._label_keywords [i]]
 
   def add_file ( self, fn ) :
       """add_file ( "filename" ) allows the user to add a plotter
       contacted via "filename" to the list of plotters being used
       by this object.
       """
       self._filename_list.append (fn)
       self._plotter_list.append (-1)
       self._plotters_started = 0

   add_display = add_file
 
   def delete_file ( self, fn ) :
       """delete_file ( "filename" ) allows the user to delete a plotter
       contacted via "filename" from the list of plotters being used
       by this object.
       """
       if fn in self._filename_list :
          self._filename_list.remove (fn)
       return

   delete_display = delete_file
 
   def add_plotter ( self, pl ) :
       """add_plotter ( pl ) allows the user to add the specified
       plotter to the list of plotters being used by this object.
       """
       self._plotter_list.append (pl)
       self._filename_list.append (" ")
 
   def delete_plotter ( self, pl ) :
       """delete_plotter ( pl ) allows the user to delete the specified
       plotter from the list of plotters being used by this object.
       """
       if pl in self._plotter_list :
          self._plotter_list.remove (pl)
       return
 
   def change ( self, * kwds , ** keywords ) :
       """change ( <keyword arguments> ) allows some of the graph's
       generic characteristics to be changed.
       Actually it allows any keyword values to be changed. It should
       be used with care, as it does no error checking.
       """
       if len (kwds) == 1 :
          keywords = kwds[0]
       self._initialize_generics ( keywords )
       for k in keywords.keys ( ) :
          setattr ( self, "_" + k , keywords [k] )

   def _init_plotter ( self ) :
       """ _init_plotter (): initializer routine which will start up
       plotter(s) if necessary. This routine is called from the plot
       routine of a derived class.
       """
       try :
          Graphics = self._graphics_list [0]
       except :
          raise self._GraphInitError, "No graphics specified??"
       if self.type () == Graph2dType :
          dpi = 100
       else :
          dpi = 75
       if len (self._plotter_list) == 0 :
          # If user has not specified a plotter, create a default
          self._filename_list.append (" ")
          self._plotter_list.append (Graphics.Plotter (" " ,
             style = self._style, dpi = dpi, hcp = self._hardcopy))
       else :
          for i in range (len (self._plotter_list)) :
              # start up plotters for any unspecified ones.
              if self._plotter_list [i] == -1 :
                 if len (self._plotter_list) > 1 :
                    Graphics = self._graphics_list [i]
                 self._plotter_list [i] = Graphics.Plotter (
                    self._filename_list [i] , style = self._style, dpi = dpi,
                    hcp = self._hardcopy)
       self._plotters_started = 1
 



#!/usr/bin/env python

import os
import sys
from glob import glob
import shutil

def configuration(parent_package='',parent_path=None):
    from scipy_distutils.core import Extension
    from scipy_distutils.misc_util import get_path,\
         default_config_dict, dot_join
    from scipy_distutils.system_info import dict_append

    package = 'special'
    config = default_config_dict(package,parent_package)
    local_path = get_path(__name__,parent_path)

    define_macros = []
    if sys.byteorder == "little":
        define_macros.append(('USE_MCONF_LE',None))
    else:
        define_macros.append(('USE_MCONF_BE',None))
    if sys.platform=='win32':
        define_macros.append(('NOINFINITIES',None))
        define_macros.append(('NONANS',None))

    c_misc = glob(os.path.join(local_path,'c_misc','*.c'))
    cephes = glob(os.path.join(local_path,'cephes','*.c'))
    if sys.platform=='win32':
        cephes = [f for f in cephes if os.path.basename(f)!='fabs.c']
    mach = glob(os.path.join(local_path,'mach','*.f'))
    amos = glob(os.path.join(local_path,'amos','*.f'))
    toms = glob(os.path.join(local_path,'toms','*.f'))
    cdf = glob(os.path.join(local_path,'cdflib','*.f'))
    specfun = glob(os.path.join(local_path, 'specfun','*.f'))
    
    # C libraries
    config['libraries'].append(('c_misc',{'sources':c_misc}))
    config['libraries'].append(('cephes',{'sources':cephes,
                                          'macros':define_macros}))

    # Fortran libraries
    config['fortran_libraries'].append(('mach',{'sources':mach}))
    config['fortran_libraries'].append(('amos',{'sources':amos}))
    config['fortran_libraries'].append(('toms',{'sources':toms}))
    config['fortran_libraries'].append(('cdf',{'sources':cdf}))
    config['fortran_libraries'].append(('specfun',{'sources':specfun}))
    
    # Extension
    sources = ['cephesmodule.c', 'amos_wrappers.c', 'specfun_wrappers.c',
               'toms_wrappers.c','cdf_wrappers.c','ufunc_extras.c']
    sources = [os.path.join(local_path,x) for x in sources]
    ext = Extension(dot_join(parent_package,package,'cephes'),sources,
                    libraries = ['amos','toms','c_misc','cephes','mach',
                                 'cdf', 'specfun'],
                    define_macros = define_macros
                    )
    config['ext_modules'].append(ext)

    ext_args = {'name':dot_join(parent_package,package,'specfun'),
                'sources':[os.path.join(local_path,'specfun.pyf')],
                'f2py_options':['--no-wrap-functions'],
                #'define_macros':[('F2PY_REPORT_ATEXIT_DISABLE',None)],
                'libraries' : ['specfun']
                }
    ext = Extension(**ext_args)
    ext.need_fcompiler_opts = 1
    config['ext_modules'].append(ext)

    return config

def get_package_config(name):
    sys.path.insert(0,os.path.join('scipy_core',name))
    try:
        mod = __import__('setup_'+name)
        config = mod.configuration()
    finally:
        del sys.path[0]
    return config

if __name__ == '__main__':
    extra_packages = []
    try: import scipy_base
    except ImportError: extra_packages.append('scipy_base')
    try: import scipy_test
    except ImportError: extra_packages.append('scipy_test')
    try: import scipy_distutils
    except ImportError:
        extra_packages.append('scipy_distutils')
        sys.path.insert(0,'scipy_core')

    from scipy_distutils.core import setup
    from scipy_distutils.misc_util import merge_config_dicts
    from special_version import special_version

    config_dict = merge_config_dicts([configuration()] + \
                                     map(get_package_config,extra_packages))

    setup(version=special_version,
          **config_dict)




#!/usr/bin/env python
import cPickle, cStringIO
import time
pickle = cPickle
#import zlib
import socket
import sys

import sync_cluster #yes I'm importing the current module

bufsize = 1<<12 #4K buffer, for Linux
shell = "ssh -X"

RemoteError = 'RemoteError'
RemoteCrashError = 'RemoteError'
PackError = 'PackError'
UnpackError = 'UnpackError'
NotImplemented = 'NotImplemented'


class pickle_packer:
    """* Pickle and unpickle an object for transfer over a socket.

        Description:
        This class takes arbitrary python objects and turns them
        into a character string suitable for transfer over a socket.
        It is currently a thin wrapper around cPickle routines, and
        thus can only pack "picklable" objects (see the python manual).
        This is not much of a limitation.

        Why do this instead of using pickle directly:
        The class is abstracted in the hopes that future packers
        might pack items much more quickly and "in place" instead of
        creating intermediate string buffers etc.  I now think a
        slightly different architecture is necessary for this to
        be possible.  The packer will need to be something like
        a "producer" in medusa.
    *"""
    def pack(self,item):
        """* Pack item and return resulting string. *"""
        try:
            packed  = pickle.dumps(item,1)
        except pickle.PicklingError, why:
            raise PackError, why
        return packed
    def unpack(self, x):
        """* Unpack string x and return the resulting object."""
        # If we get an empty string back, more than likely
        # the remote worker crashed. - They are always supposed
        # to send back something.
        #if x == '':
        #    raise RemoteCrashError
        try:
            res = pickle.loads(x)
        except pickle.UnpicklingError, why:
            raise UnpackError, why
        return res

    def fpack(self,item,f):
        """* Pack item into the file object f. Nothing is returned.
            f can also be a cStringIO object. fpack can be
            called multiple times with the same file object
            to pack several items into f.
        *"""
        try:
            packed  = pickle.dumps(item,1)
        except pickle.PicklingError, why:
            raise PackError, why

    def funpack(self, f):
        """* Unpack an object from thr file object f.  The
            resulting object is returned. f can also be
            a cStringIO object.  if multiple items are
            packed within f, funpack can be called multiple
            times to retreive the objects.
        *"""
        try:
            res = pickle.load(f)
        except pickle.UnpicklingError, why:
            raise UnpackError, why
        return res

class blocking_channel:
    """* Manages sending and receiving data through a blocking socket.

        Description:
        The channel is meant for a single exchange of data between
        two programs.  Generally, data is sent down the write channel
        (wfile) using write().  The write channel is then closed
        with close_write() signally to the other end that all
        data has been sent.  The program on the other end then reads
        the data, processes it, and sends a response back down the
        read channel (rfile).  You can read the the results using recv().

        Many programs, like chat, keep a socket open for many exchanges.
        This is possible when sending text data back and forth because
        you can indicate the end of a sent message with a delimiter
        character.  When sending binary data, however, it is not possible
        to break up messages with a delimiter because the delimiter
        might occur randomly within the binary data indicating a false
        message break.  Closing the write channel is an unambiguous
        end of message indicator.

        This channel is used by the standard_client to send snippets
        of python code along with pickled objects to a remote machine.
        The remote machine then executes the code and returns the results
        to the original machine.

        Because the underlying sockets are blocking, this method may not
        be the most efficient way to communicate with many different
        clients.  Non-blocking sockets, as are used in medusa, may
        be more efficient, but the programming model is slightly more
        confusing.  See ***.
    *"""
    def __init__(self,host,port,log = None):
        """* Open a channel (socket) to a remote machine.  The
             socket attempts to connect to the host on the
             indicated port.
        *"""
        self.sock = socket.socket( socket.AF_INET, socket.SOCK_STREAM)
        #print 'connecting: ', host, port
        self.sock.connect((host,port))
        self.host,self.port = host,port
        #print 'creating files'
        self.rfile = self.sock.makefile('rb', bufsize)
        self.wfile = self.sock.makefile('wb', bufsize)
        import sys
        self.log = log
        #self.log = sys.stdout
    def write(self,x):
        """* Send data to the remote machine.  This function
            can be called multiple times.  The net affect, though
            is that all data sent is concatenated together to
            form a single message read on the other end.

            The remote machine will not read any of the data sent
            until close_write() is called to signal that all data
            is sent.  Calls to write() after close_write() has
            been called will result in an error.
        *"""
        self.log_msg( ("writing %d bytes" % len(x)) )
        self.wfile.write(x)
    def close_write(self):
        """* Close the write channel.  This signals that all data
             has been sent to the remote end of the connection.
             Further calls to write() will result in an error.
        *"""
        self.log_msg("closing write channel")
        self.wfile.flush()
        self.wfile.close()
        self.sock.shutdown(1) #tells other side transfer is over
    def read(self):
        """* Return a string of data returned by the remote connection.
            read() should not be called until after close_write(). It
            is a blocking call and will wait indefinitely if the
            remote connection does not send back data and close the
            connection.

            read() stops reading data when the rfile is closed
            by remote connection signally the end of data.  It then
            closes the socket.  read() can only be called once.
            Further calls will result in an error.
        *"""
        x = self.rfile.read()
        self.log_msg( ("read %d bytes" % len(x)) )
        self.rfile.close()
        self.sock.close()
        return x
    def write_read(self,data):
        """* Send data to the remote connection, wait for it to
            process the data, and return a response.  The
            response is returned as a string.  After calling this
            function the socket is closed and further calls
            to write(), read() or wrtie_read() will result in
            errors.
        *"""
        sent = self.write(data)
        self.close_write()
        return self.read()
    def log_msg(self,msg):
        if self.log:
            pre_msg = 'blocking_channel on %s, %d:' % (self.host,self.port)
            self.log.write(pre_msg+msg+'\n')
            self.log.flush()

class standard_sync_client:
    """* Client side of a remote Python interpreter.

        This class allows you to connect to remote machines
        and send them arbitrary python code and objects.
        The remote machine executes the code and sends it
        back to you.  It is intended to work with
        standard_sync_handler on the server side.  Together,
        they make a powerful building block for "embarassingly
        parallel" distributed computing.  Note, however,
        that this class used blocking sockets (hence the
        "sync" in the name).  A non-blocking client
        may lead to more efficient use of available bandwidth.

        Note that, when working with a cluster of computers,
        using wscluster is much easier than trying to manage
        the machine client on your own.  It handles splitting
        up the processing across a cluster of workstations much
        more transparently.

        There are several ways to used standard_sync_client:

            1) As a dictionary for setting and getting
               global variables on a remote interpreter.
            2) To execute a function in a similar fashion
               to calling apply()
            3) To execute a code fragment similar to
               calling exec
            4) loop() calls the same function with multiple
               different inputs. (SIMD)

        This functionality is accessible in an two different
        formats, "easy to use" or "most efficient".  ** something
        about load balancing statistics here **

        The client and server machines are assumed to have
        identical copies of all module files.  Also, any
        object, function or class sent to remote machines
        must be defined in a module.  Objects are "pickled"
        before being sent and pickling objects
        from __main__ causes problems when unpickling
        them on the remote end.

        apply() example:
        Suppose you have a function process_image() that
        processes an image and returns some result.

            # in module radar.py
            # ** make this a real function
            # that does something time consuming **
            def process_image(image):
                H = fft2(image)
                # some other expensive operations
                return result

       Here's one way to execute this on remote machines.
       This assumes a server is up and running on the machines.

            # 1. Create an 256x256 image of random pixels
            #    between 0 and 1.
            import scipy.stats 
            image1 = scipy.stats.uniform(0.,1.,(256,256))
            image2 = scipy.stats.uniform(0.,1.,(256,256))
            # 2. Create connections to remote machines
            #    named cow3 and cow4 on port 10000
            remote1 = standard_sync_client('cow3',10000)
            remote2 = standard_sync_client('cow4',10000)
            # 3. Call apply() on the remote machine.
            result1 = remote1.apply(radar.process_image,(image1,))
            result2 = remote2.apply(radar.process_image,(image2,))

        The remote apply() has identical calling semantics
        to the standard one.  See the primer (below) on the standard
        Pyton apply for more insight into how this function works.

        if you time the previous calls, you'll note that they don't
        occur twice as fast simply because they are processed
        on two different machines.  Calls to apply()
        are blocking.  Image1 is processed completely before
        image2 is processed here.  More than likely you'd rather
        the work happen in parallel.  This requires you to
        separate the read and write processes.

            # 3. Call apply() on the remote machine.
            #   a) call apply pack to prepare the messages.
            #   b) send to each machine.
            #   c) read from each machine.
            task1 = remote1.apply_pack(radar.process_image,(image1,))
            task2 = remote2.apply_pack(radar.process_image,(image2,))
            remote1.send(task1)
            remote2.send(task1)
            result1 = remote1.recv(task1)
            result2 = remote1.recv(task2)

        The send and recv are still blocking, but if the processing
        time for the images are much larger than the packing and
        transmit time, you'll get your images processed in about
        half the time.

        exec() example:
        Executing code fragments proceeds in a similar fashion.
        It is modeled after the exec statement, but takes
        more parameters as a consequence of being remote.
        Additionly, exec is a function here instead of a statement.
        # **more here**

        apply() primer:
        The apply() function in Python is used to "apply"
        a set of arguments to a function takes three arguements.
        The first is a function, the second is a tuple
        of arguments, and the third is a dictionary of
        keyword arguements. for example if the function is:

            def silly_func(a,b,c,d=2):
                return a+b+c+d

       then apply can be called by passing all arguments
       in through the argument list

           >> r = apply(silly_func,(1,2,3,4))
           r = 10

        or some can be passed in using the argument list
        and others using keywords

           >> r = apply(silly_func,(1,),{'b':2,'c':3,'d':4})
           r = 10

        Note the "," after 1 here. The second
        argument must be tuple.  Python
        interprets (1) as the number 1 and (1,) as
        a tuple with 1 as its only entry.  Such
        silliness goes away if the function takes
        more than one arguement.

        See the standard Python tutorial for more
        info on calling functions with apply().
    *"""
    def __init__(self,host,port,my_id=-1,log=None):
        self.packer = pickle_packer()
        self.host,self.port = host,port
        self.id = my_id
        self.log = log
    def start_server(self):
        """* expands to something like:
        rsh -n 127.0.0.1 "python ~/wrk/sync_cluster.py server 10000 >&/dev/null </dev/null &"
        *"""
        import sync_cluster, os
        module_name = os.path.abspath(sync_cluster.__file__)
        #cmd = 'rsh -n %s "python %s server %d >&/dev/null </dev/null &"' % \
        #      (self.host, module_name, self.port)
        cmd = '%s %s "python %s server %d >&/tmp/crud%d </dev/null &"' % \
              (shell,self.host, module_name, self.port, self.port)
        self.log_msg(cmd)
        #print cmd
        os.system(cmd)

    def exec_code_pack(self,code,inputs=None,returns=None,global_vars=None):
        contents={'_command':sync_cluster.exec_code,'code':code,
                  'inputs':inputs,'returns':returns,
                  'global_vars':global_vars }
        return self.packer.pack(contents)

    def loop_code_pack(self,code,loop_var,inputs=None,returns=None,global_vars=None):
        contents={'_command':sync_cluster.loop_code,'code':code,
                  'loop_var':loop_var,'inputs':inputs,
                  'returns':returns,'global_vars':global_vars }
        return self.packer.pack(contents)
    def apply_pack(self,function,args,keywords=None):
        if not keywords: keywords = {}
        #more_keywords = args_to_keywords(function,args)
        #catch_keyword_conflicts(more_keywords,keywords)
        #keywords.update(more_keywords)
        contents={'_command':sync_cluster.apply_func,'function':function,
                  'args':args,'keywords':keywords}
        return self.packer.pack(contents)
    def loop_apply_pack(self,function,loop_var,args,keywords=None):
        # NOT USED -- see cow.machine_cluster.loop_apply.
        all_keywords = {}
        if keywords: all_keywords.update(keywords)
        more_keywords = args_to_keywords(function,args)
        catch_keyword_conflicts(more_keywords,all_keywords)
        all_keywords.update(more_keywords)
        contents={'_command':sync_cluster.loop_func,'function':function,
                  'keywords':all_keywords,'loop_var':loop_var}
        return self.packer.pack(contents)

    def apply(self,function,args,keywords=None):
        package = self.apply_pack(function,args,keywords)
        self.send(package)
        return self.recv()
        
    def exec_code(self,code,inputs=None,returns=None,global_vars=None):
        package = self.exec_code_pack(code,inputs,returns,global_vars)
        self.send(package)
        return self.recv()

    def loop_apply(self,function,loop_var,args,keywords=None):
        # NOT USED -- see cow.machine_cluster.loop_apply.
        package = self.loop_apply_pack(function,loop_var,args,keywords)
        self.send(package)
        return self.recv()

    def loop_code(self,code,loop_var,inputs=None,returns=None,global_vars=None):
        # NOT USED -- see cow.machine_cluster.loop_code.
        package = self.loop_code_pack(code,loop_var,inputs,returns,global_vars)
        self.send(package)
        return self.recv()
    def send(self,package,addendum=None):
        """* addendum is a dictionary.  It is not packed...*"""
        # build a socket
        self.channel = blocking_channel(self.host,self.port)
        self.channel.write(package)
        if addendum:
            extra_package = self.packer.pack(addendum)
            self.channel.write(extra_package)
        self.channel.close_write()
    def recv(self):
        package = self.channel.read()
        contents = self.packer.unpack(package)
        #print 'self.id, ',self.id, contents
        self.catch_exception(contents)
        #print 'after exception'
        #try:    print  contents['_exec_time']
        #except: pass    
        return contents['result']
    def get_load_info(contents):
        #use this to read execution time info from the package...
        pass
    def log_msg(self,msg):
        if self.log:
            pre_msg = 'client %d:' % self.id
            self.log.write(pre_msg+msg+'\n')
            self.log.flush()

    #-------------------------------------------
    # Partial mimicking of a Dictionary.
    # Need some thought on how to deal with keys(),
    # items(), len(), clear(), and values()
    #
    # Only strings can be used as keys.
    #-------------------------------------------

    def get_pack(self,keys):
        contents={'_command':sync_cluster.get_keys,
                  'keys':keys}
        return self.packer.pack(contents)
    def set_pack(self,key,item):
        # relies on update_pack.
        contents={'_command':sync_cluster.update,
                  'global_dict':{key:item}}
        return self.packer.pack(contents)
    def update_pack(self,global_dict):
        contents={'_command':sync_cluster.update,
                  'global_dict':global_dict}
        return self.packer.pack(contents)
    def del_pack(self,keys):
        contents={'_command':sync_cluster.del_keys,
                  'keys':keys}
        return self.packer.pack(contents)

    def __getitem__(self, key):
        # currently allowing tuples also!
        #assert(type(key) is type(''))
        package = self.get_pack(key)
        self.send(package)
        return self.recv()

    def __setitem__(self, key, item):
        assert(type(key) is type(''))
        package = self.update_pack({key:item})
        self.send(package)
        self.recv() #must receive to catch exceptions

    def __delitem__(self, key):
        # currently allowing tuples also!
        # assert(type(key) is type(''))
        package = self.del_pack(key)
        self.send(package)
        self.recv() #must receive to catch exceptions

    def update(self, dict):
        package = self.update_pack(dict)
        self.send(package)
        self.recv() #must receive to catch exceptions

    def clear(self):raise NotImplemented
    def keys(self): raise NotImplemented
    def items(self): raise NotImplemented
    def values(self): raise NotImplemented
    def has_key(self, key): raise NotImplemented
    def get(self, key, failobj=None): raise NotImplemented

    #-------------------------------------------
    # End of dictionary.
    #-------------------------------------------

    def catch_exception(self,x):
        """* exception_traceback is really a string representation
            of the error.  The actual traceback can't be sent
            because of pickleing issues.
        *"""
        try:
            err = x['type'],x['msg'],x['exception_traceback']
            raise RemoteError, err
        except (TypeError, KeyError, AttributeError): pass

def args_to_keywords(function,args):
    arg_keywords = {}
    N = len(args)
    for i in range(N):
        arg_name = function.func_code.co_varnames[i]
        arg_keywords[arg_name] = args[i]
    return arg_keywords

def catch_keyword_conflicts(kw1, kw2):
    keys1 = kw1.keys()
    keys2 = kw2.keys()
    for key in keys1:
        if key in keys2:
            raise TypeError, ('keyword parameter "%s" redefined' % key)

def read_log():
    """ Read the results of the log file.  
    
        This assumes that stdout and the log file are the same.    
    """
    #global log_file
    #sys.stdout.close()
    #f = open(log_file,'r')
    #results = f.read(-1)
    #f.close()
    #sys.stdout = open(log_file,'a+')
    #return results
    pass
    
import SocketServer
class standard_sync_handler(SocketServer.StreamRequestHandler):
    verbose = 1
    packer = pickle_packer()
    def setup(self):
        """* This function is to fix a bug in the python library.
             The newest release of Python should have that bug fixed,
             and this function can be removed
        *"""
        self.connection = self.request
        print 'here:',self.connection
        self.rfile = self.connection.makefile('rb', bufsize)
        self.wfile = self.connection.makefile('wb', bufsize)
        
        #SocketServer.StreamRequestHandler.setup(self)
        #import tempfile,os
        #global log_file
        #dr = tempfile.gettempdir()
        #log_file = os.path.join(dr,'cow.log')
        #f = open(log_file,'w')
        #sys.stdout = f
        self.log = sys.stdout

    def handle(self):
        import time
        kill_when_finished = 0
        try:
            recv_msg = self.recv()
            #print 'received msg'
            send_msg = self.process(recv_msg)
            #print 'processed msg'
            packed_msg = self.packer.pack(send_msg)
            if self.verbose: print '\tserver: sending %d bytes' % len(packed_msg)
            #print 'send msg:', send_msg
        except SystemExit:
            import sys
            err = sys.exc_info()
            packed_msg = self.pack_exception(err)
            del err
            kill_when_finished = 1
            #print 'should exit'
        except:
            import sys
            err = sys.exc_info()
            packed_msg = self.pack_exception(err)
            #print 'packed exception:', err[0],err[1]
            del err

        self.wfile.write(packed_msg)
        #print 'sent msg'
        #print '\tserver: sent %d bytes' % len(packed_msg)
        if self.verbose: print '\tserver: sent %d bytes' % len(packed_msg)
        if kill_when_finished:
            print 'killing'
            self.finish() # make sure this socke this closed
            #signal main server to die
            import sys,os
            #ppid = os.getppid ()
            #print ppid
            force_kill()
            sys.exit(0)
        #print 'kill_when_finished', kill_when_finished

    def finish(self):
        """* This function shouldn't have to be defined, but, on occasion,
            when an Exception is thrown in the code processed by loop_code
            ( and maybe others ), the socket (request) doesn't die, even
            when all the files are closed.  As a result, the other end
            blocks forever.  We close the socket explicitly here, until
            we find the real problem.  The exceptions causing the problems
            are occuring in Fortran routines, but the execptions seem
            to be caught properly on this end and sent down the channel.
        *"""
        self.wfile.flush()
        self.wfile.close()
        self.rfile.close()
        self.request.close() # hmm.  Shouldn't need this, but occasionaly seems necessary.
        #print 'should be closed'

    def recv(self):
        if self.verbose: print '\tserver: connection established'
        #need to do something here to handle extra messages...
        packed_msg =self.rfile.read()
        if self.verbose: print '\tserver: received %d bytes' % len(packed_msg)

        packed_msg = cStringIO.StringIO(packed_msg)
        task = self.packer.funpack(packed_msg)
        current_location = packed_msg.tell()
        packed_msg.seek(0,2)
        end_location = packed_msg.tell()
        if(end_location != current_location):
            packed_msg.seek(current_location)
            addendum = self.packer.funpack(packed_msg)
        else:
            addendum = None
        task['addendum'] = addendum
        return task

    def process(self,task):
        # find out the command type (this is really a function)
        command = task['_command']
        # remove the command from the task.  The task now
        # represents
        del task['_command']
        t1 = time.time()
        result= apply(command,(),task)
        t2 = time.time()
        result_dict = {'result':result,'_exec_time':t2-t1}
        #add load info here
        return result_dict
    def pack_exception(self,err):
        import traceback,string
        msg = {'exception_traceback':
               '########### Traceback text from remote machine ###########\n' \
               + string.join(traceback.format_exception(err[0],err[1],err[2]),'') \
               + '################# End remote traceback ##################',
               'type': err[0],
               'msg': err[1]}
        return self.packer.pack(msg)

    def log_msg(self,msg):
        if self.log_name:
            pre_msg = 'client %d:' % self.id
            self.log.write(pre_msg+msg+'\n')

#there may be some issues with globals scope in these
def apply_func(function, args, keywords=None,addendum=None):
    if not keywords: keywords = {}
    if addendum: keywords.update(addendum)
    return apply(function,args ,keywords)

def exec_code(code,inputs,returns,global_vars,addendum=None):
    if addendum: inputs.update(addendum)
    if not returns: returns = ()
    if type(returns) == type(''):
        raise TypeError, 'returns must be a sequence object - not a string'
    exec_code = build_globals(global_vars)
    exec_code = exec_code + build_inputs(inputs)
    exec_code = exec_code + code
    globals()['_inputs'] = inputs
    exec exec_code in globals(), globals()
    #perhaps do something here to catch errors
    if len(returns) == 1:
        results = eval(returns[0])
    elif len(returns) > 1:
        results = []
        for i in returns:
            results.append(eval(i))
        results = tuple(results)
    else:
        results = None
    return results

def loop_func(function,loop_var,args,keywords,addendum=None):
    if not keywords: keywords = {}
    if addendum: 
        keywords.update(addendum)
    result = []
    _loop_data = keywords[loop_var]
    del keywords[loop_var] #not strictly necessary
    if type(loop_var) == type(''):                                
        for _i in _loop_data:
            keywords[loop_var] = _i
            result.append(apply(function,args ,keywords))
    elif type(loop_var) == type(1):
        args_list = list(args)
        for _i in _loop_data:
            args_list[loop_var] = _i
            args = tuple(args_list)
            result.append(apply(function,args,keywords))
    return tuple(result)

def loop_code(code,loop_var,inputs,returns,global_vars,addendum=None):
    if type(returns) == type(''):
        raise TypeError, 'returns must be a sequence object - not a string'
    if addendum: inputs.update(addendum)
    globals()['_loop_data'] = inputs[loop_var]
    globals()['_returns'] = returns 
    #added to set all inputs in the global namespace
    globals().update(inputs)
    del inputs[loop_var] #not strictly necessary
    exec_code = build_loop_code(code,loop_var,inputs,returns,global_vars)
    exec exec_code in globals(), globals()
    return _all_results

#------------------------------------------------
# dictionary like routines for global variables.
# All ignore addendum.
#------------------------------------------------
def set_key(key,item,addendum=None):
    globals()[key] = item

def update(global_dict,addendum=None):
    globals().update(global_dict)

def get_keys(keys,addendum=None):
    if type(keys) == type(''):
        return globals()[keys]
    else:
        results = []
        for i in keys:
            results.append(globals()[i])
        return tuple(results)

def del_keys(keys,addendum=None):
    if type(keys) == type(''):
        del globals()[keys]
    else:
        for i in keys:
            del globals()[i]

#------------------------------------------------
# utilities and template strings used for building
# loop_code and exec_code strings.
#------------------------------------------------

import string

def build_globals(global_vars):
    # if we have global variables specified, we
    # need to append there definition:
    #    global a, b, c, etc.
    # at the beginning of the code to be executed
    # This ensure that global variables set by the
    # code will actually be set globally instead
    # of locally
    if global_vars:
        return 'global %s;' % string.join(global_vars,',')
    else:
        return ''

def build_inputs(inputs):
    if inputs:
        asgn = map(lambda x: '%s = _inputs["%s"];' % (x,x), inputs.keys())
        return string.join(asgn,'')
    else:
        return ''

def indent(code):
    import string
    code = string.replace(code,'\n','\n    ')
    return code

loop_code_template = """
%(global_code)s
%(input_code)s
_all_results = []
for %(loop_var)s in _loop_data:
    %(code)s
    _result = []
    if len(_returns) == 1:
        _result = eval(_returns[0])
    elif len(_returns) > 1:
        _result = []
        for _j in _returns:
            _result.append(eval(_j))
        _result = tuple(_result)
    else:
        _result = None
    _all_results.append(_result)
_all_results = tuple(_all_results)
"""

def build_loop_code(code,loop_var,inputs,returns,global_vars):
    code_entries ={}
    code_entries['global_code'] = '' # build_globals(global_vars)
    code_entries['input_code'] = '' # build_inputs(inputs)
    code_entries['loop_var'] = loop_var
    code_entries['code'] = indent(code)
    exec_code = loop_code_template % code_entries
    return exec_code
#------------------------------------------------
server_pid = 0
def force_kill():
    """* this is silliness, but I couldn't figure out
        a more elegant approach to killing the server
        process. calling os.exit() in the handler
        only seemed to kill the handler thread -
        not the listening thread
    *"""
    global server_pid
    import os
    print server_pid
    if os.name == 'nt':
        import win32api
        ph = win32api.OpenProcess(1,0,server_pid)
        win32api.TerminateProcess(ph,0)
    else:
        os.kill(server_pid,15) # 15 = TERM 9 = ABRT

class MyThreadingTCPServer(SocketServer.ThreadingTCPServer):
    """ Threaded Server for handling request for python commands
    
        This class was added as of Python2.1 in response
        to the bug reported at:
           
           http://sourceforge.net/tracker/index.php?
              func=detail&aid=417845&group_id=5470&atid=105470
        
        Hopefully the issue will get fixed in the standard library
        by 2.2, and this can go away.
    """
    def handle_request(self):
        """Handle one request, possibly blocking."""
        try:
            request, client_address = self.get_request()
        except socket.error:
            return
        if self.verify_request(request, client_address):
            try:
                self.process_request(request, client_address)
            except:
                self.handle_error(request, client_address)
                self.close_request(request)

    def finish_request(self, request, client_address):
        """Finish one request by instantiating RequestHandlerClass."""
        print 'finish request:', request
        self.RequestHandlerClass(request, client_address, self)
        self.close_request(request)

default_host = socket.gethostname()
def server(host=default_host,port=10000):
    import os
    global server_pid
    server_pid = os.getpid()
    sync_cluster.server_pid = server_pid
    print "starting server on %s:%s" % (host,port)
    print server_pid
    #the_server=SocketServer.TCPServer( (host, port), standard_sync_handler)
    #the_server=SocketServer.ForkingTCPServer( (host, port), standard_sync_handler)
    the_server=MyThreadingTCPServer( (host, port), standard_sync_handler)
    __name__ = '__main__'
    the_server.serve_forever()

def client(host,port,length=1e5):
    import time
    t1 = time.time()
    machine = standard_sync_client(host,port)
    print 'sock open time: ', time.time() - t1

    func = sync_cluster.test_function
    args = (1,2)
    t1 = time.time()
    addendum = {'b':4}
    task = machine.apply_pack(func,args)
    machine.send(task,addendum)
    results = machine.recv()
    t2 = time.time()

#   print 'sent: (beginning)', msg[:5]
#   print 'recv: (beginning)',response[:5]
    print 'roundtrip time:', t2-t1, 'sec'
    #print t2 - t1, (8*2*float(len(msg))/(t2-t1)    / 1e6)

usage = """
sync_cluster mode port host length
mode: server, daemon, client, test
port: server or daemon - port to start server on
      client or test - port to connect to
host - client or test - machine to connect to
length = client - length of message to send
"""

host = default_host
port = 10000

if __name__ == '__main__':
    import sys
    import string
    host = default_host
    if len(sys.argv) >= 5: length = string.atoi(sys.argv[4])
    if len(sys.argv) >= 4: host = sys.argv[3]
    if len(sys.argv) >= 3: port = string.atoi(sys.argv[2])
    if len(sys.argv) >= 2:
        if sys.argv[1] == 'server':
            server(host,port)
        elif sys.argv[1] == 'daemon':
            import os, sync_cluster
            pid = os.fork()
            if pid==0:
                print sync_cluster.__file__, port
                # Child process.
                # This must never return, hence os._exit()!
                try:
                    os.setsid()
                    server(host,port)
                    print "yikes - shouldn't get here"
                finally:
                    os._exit(1)
            else:
                #parent process
                print 'parent'
                os._exit(0)

        elif sys.argv[1] == 'client':
            client(host,port)
        elif sys.argv[1] == 'test':
            import sync_cluster_test
            sync_cluster_test.test(host,port)
        else: print usage
    else:
        print usage
    print 'done'



""" Test refcounting and behavior of SCXX.
"""
import unittest
import time
import os,sys

from scipy_test.testing import *
set_package_path()
from weave import inline_tools
restore_path()


class test_dict_construct(unittest.TestCase):
    #------------------------------------------------------------------------
    # Check that construction from basic types is allowed and have correct
    # reference counts
    #------------------------------------------------------------------------
    def check_empty(self,level=5):
        # strange int value used to try and make sure refcount is 2.
        code = """
               py::dict val;
               return_val = val;
               """
        res = inline_tools.inline(code)
        assert sys.getrefcount(res) == 2
        assert res == {}
                              
                    
class test_dict_has_key(unittest.TestCase):
    def check_obj(self,level=5):
        class foo:
            pass
        key = foo()
        a = {}
        a[key] = 12345
        code = """
               return_val = a.has_key(key);               
               """
        res = inline_tools.inline(code,['a','key'])
        assert res
    def check_int(self,level=5):
        a = {}
        a[1234] = 12345
        code = """
               return_val = a.has_key(1234);               
               """
        res = inline_tools.inline(code,['a'])
        assert res
    def check_double(self,level=5):
        a = {}
        a[1234.] = 12345
        code = """
               return_val = a.has_key(1234.);               
               """
        res = inline_tools.inline(code,['a'])
        assert res
    def check_complex(self,level=5):
        a = {}
        a[1+1j] = 12345
        key = 1+1j
        code = """
               return_val = a.has_key(key);               
               """
        res = inline_tools.inline(code,['a','key'])
        assert res
        
    def check_string(self,level=5):
        a = {}
        a["b"] = 12345
        code = """
               return_val = a.has_key("b");               
               """
        res = inline_tools.inline(code,['a'])
        assert res
    def check_std_string(self,level=5):
        a = {}
        a["b"] = 12345
        key_name = "b"
        code = """
               return_val = a.has_key(key_name);               
               """
        res = inline_tools.inline(code,['a','key_name'])
        assert res        
    def check_string_fail(self,level=5):
        a = {}
        a["b"] = 12345
        code = """
               return_val = a.has_key("c");               
               """
        res = inline_tools.inline(code,['a'])
        assert not res

class test_dict_get_item_op(unittest.TestCase):

    def generic_get(self,code,args=['a']):
        a = {}
        a['b'] = 12345
                
        res = inline_tools.inline(code,args)
        assert res == a['b']

    def check_char(self,level=5):
        self.generic_get('return_val = a["b"];')

    def DOESNT_WORK_check_char_fail(self,level=5):
        # We can't through a KeyError for dicts on RHS of
        # = but not on LHS.  Not sure how to deal with this.
        try:
            self.generic_get('return_val = a["c"];')
        except KeyError:
            pass
            
    def check_string(self,level=5):
        self.generic_get('return_val = a[std::string("b")];')


    def check_obj(self,level=5):
        code = """
               py::object name = "b";
               return_val = a[name];
               """ 
        self.generic_get(code,['a'])

    def DOESNT_WORK_check_obj_fail(self,level=5):
        # We can't through a KeyError for dicts on RHS of
        # = but not on LHS.  Not sure how to deal with this.
        try:
            code = """
                   py::object name = "c";
                   return_val = a[name];
                   """ 
            self.generic_get(code,['a'])
        except KeyError:
            pass    
            
class test_dict_set_operator(unittest.TestCase):
    def generic_new(self,key,val):
        # test that value is set correctly and that reference counts
        # on dict, key, and val are being handled correctly.
        a = {}
        # call once to handle mysterious addition of one ref count
        # on first call to inline.
        inline_tools.inline("a[key] = val;",['a','key','val'])
        assert a[key] == val
        before = sys.getrefcount(a), sys.getrefcount(key), sys.getrefcount(val)
        inline_tools.inline("a[key] = val;",['a','key','val'])
        assert a[key] == val
        after = sys.getrefcount(a), sys.getrefcount(key), sys.getrefcount(val)
        assert before == after
    def generic_overwrite(self,key,val):
        a = {}
        overwritten = 1
        a[key] = overwritten # put an item in the dict to be overwritten
        # call once to handle mysterious addition of one ref count
        # on first call to inline.
        before_overwritten = sys.getrefcount(overwritten)
        inline_tools.inline("a[key] = val;",['a','key','val'])
        assert a[key] == val
        before = sys.getrefcount(a), sys.getrefcount(key), sys.getrefcount(val)
        inline_tools.inline("a[key] = val;",['a','key','val'])
        assert a[key] == val
        after = sys.getrefcount(a), sys.getrefcount(key), sys.getrefcount(val)
        after_overwritten = sys.getrefcount(overwritten)
        assert before == after
        assert before_overwritten == after_overwritten
        
    def check_new_int_int(self,level=5):
        key,val = 1234,12345
        self.generic_new(key,val)
    def check_new_double_int(self,level=5):
        key,val = 1234.,12345
        self.generic_new(key,val)
    def check_new_std_string_int(self,level=5):
        key,val = "hello",12345
        self.generic_new(key,val)
    def check_new_complex_int(self,level=5):
        key,val = 1+1j,12345
        self.generic_new(key,val)
    def check_new_obj_int(self,level=5):
        class foo:
           pass
        key,val = foo(),12345
        self.generic_new(key,val)

    def check_overwrite_int_int(self,level=5):
        key,val = 1234,12345
        self.generic_overwrite(key,val)
    def check_overwrite_double_int(self,level=5):
        key,val = 1234.,12345
        self.generic_overwrite(key,val)
    def check_overwrite_std_string_int(self,level=5):
        key,val = "hello",12345
        self.generic_overwrite(key,val)
    def check_overwrite_complex_int(self,level=5):
        key,val = 1+1j,12345
        self.generic_overwrite(key,val)
    def check_overwrite_obj_int(self,level=5):
        class foo:
           pass
        key,val = foo(),12345
        self.generic_overwrite(key,val)
                
class test_dict_del(unittest.TestCase):
    def generic(self,key):
        # test that value is set correctly and that reference counts
        # on dict, key, are being handled correctly. after deletion,
        # the keys refcount should be one less than before.
        a = {}
        a[key] = 1
        inline_tools.inline("a.del(key);",['a','key'])
        assert not a.has_key(key)
        a[key] = 1
        before = sys.getrefcount(a), sys.getrefcount(key)
        inline_tools.inline("a.del(key);",['a','key'])
        assert not a.has_key(key)
        after = sys.getrefcount(a), sys.getrefcount(key)
        assert before[0] == after[0]
        assert before[1] == after[1] + 1
    def check_int(self,level=5):
        key = 1234
        self.generic(key)
    def check_double(self,level=5):
        key = 1234.
        self.generic(key)
    def check_std_string(self,level=5):
        key = "hello"
        self.generic(key)
    def check_complex(self,level=5):
        key = 1+1j
        self.generic(key)
    def check_obj(self,level=5):
        class foo:
           pass
        key = foo()
        self.generic(key)

class test_dict_others(unittest.TestCase):
    def check_clear(self,level=5):
        a = {}
        a["hello"] = 1
        inline_tools.inline("a.clear();",['a'])
        assert not a
    def check_items(self,level=5):
        a = {}
        a["hello"] = 1
        items = inline_tools.inline("return_val = a.items();",['a'])
        assert items == a.items()
    def check_values(self,level=5):
        a = {}
        a["hello"] = 1
        values = inline_tools.inline("return_val = a.values();",['a'])
        assert values == a.values()
    def check_keys(self,level=5):
        a = {}
        a["hello"] = 1
        keys = inline_tools.inline("return_val = a.keys();",['a'])
        assert keys == a.keys()
    def check_update(self,level=5):
        a,b = {},{}
        a["hello"] = 1
        b["hello"] = 2
        inline_tools.inline("a.update(b);",['a','b'])
        assert a == b

if __name__ == "__main__":
    ScipyTest('weave.scxx').run()




import sys
from distutils.core import Command

#XXX: Implement confic_cc for enhancing C/C++ compiler options.

class config_fc(Command):
    """ Distutils command to hold user specified options
    to Fortran compilers.

    config_fc command is used by the FCompiler.customize() method.
    """

    user_options = [
        ('fcompiler=',None,"specify Fortran compiler type"),
        ('f77exec=', None, "specify F77 compiler command"),
        ('f90exec=', None, "specify F90 compiler command"),
        ('f77flags=',None,"specify F77 compiler flags"),
        ('f90flags=',None,"specify F90 compiler flags"),
        ('opt=',None,"specify optimization flags"),
        ('arch=',None,"specify architecture specific optimization flags"),
        ('debug','g',"compile with debugging information"),
        ('noopt',None,"compile without optimization"),
        ('noarch',None,"compile without arch-dependent optimization"),
        ('help-fcompiler',None,"list available Fortran compilers"),
        ]

    boolean_options = ['debug','noopt','noarch','help-fcompiler']

    def initialize_options(self):
        self.fcompiler = None
        self.f77exec = None
        self.f90exec = None
        self.f77flags = None
        self.f90flags = None
        self.opt = None
        self.arch = None
        self.debug = None
        self.noopt = None
        self.noarch = None
        self.help_fcompiler = None
        return

    def finalize_options(self):
        if self.help_fcompiler:
            from scipy_distutils.fcompiler import show_fcompilers
            show_fcompilers(self.distribution)
            sys.exit()
        return

    def run(self):
        # Do nothing.
        return





#
# stats - Statistical Functions
#

from info_stats import __doc__

import pstats
from stats import *
from distributions import *
from rv import *
from morestats import *

try:  # use R functions if installed.
    import rpy
    from rfuncs import *
except ImportError:
    pass




import types
import Numeric
__all__ = ['mgrid','ogrid','r_','c_','index_exp']

from type_check import ScalarType, asarray
import function_base
import matrix_base

class nd_grid:
    """ Construct a "meshgrid" in N-dimensions.

        grid = nd_grid() creates an instance which will return a mesh-grid
        when indexed.  The dimension and number of the output arrays are equal
        to the number of indexing dimensions.  If the step length is not a
        complex number, then the stop is not inclusive.
    
        However, if the step length is a COMPLEX NUMBER (e.g. 5j), then the
        integer part of it's magnitude is interpreted as specifying the
        number of points to create between the start and stop values, where
        the stop value IS INCLUSIVE.

        If instantiated with an argument of 1, the mesh-grid is open or not
        fleshed out so that only one-dimension of each returned argument is
        greater than 1
    
        Example:
    
           >>> mgrid = nd_grid()
           >>> mgrid[0:5,0:5]
           array([[[0, 0, 0, 0, 0],
                   [1, 1, 1, 1, 1],
                   [2, 2, 2, 2, 2],
                   [3, 3, 3, 3, 3],
                   [4, 4, 4, 4, 4]],
                  [[0, 1, 2, 3, 4],
                   [0, 1, 2, 3, 4],
                   [0, 1, 2, 3, 4],
                   [0, 1, 2, 3, 4],
                   [0, 1, 2, 3, 4]]])
           >>> mgrid[-1:1:5j]
           array([-1. , -0.5,  0. ,  0.5,  1. ])

           >>> ogrid = nd_grid(1)
           >>> ogrid[0:5,0:5]
           [array([[0],[1],[2],[3],[4]]), array([[0, 1, 2, 3, 4]])] 
    """
    def __init__(self, sparse=0):
        self.sparse = sparse
    def __getitem__(self,key):
        try:
	    size = []
            typecode = Numeric.Int
	    for k in range(len(key)):
	        step = key[k].step
                start = key[k].start
                if start is None: start=0
                if step is None: step=1
                if type(step) is type(1j):
                    size.append(int(abs(step)))
                    typecode = Numeric.Float
                else:
                    size.append(int((key[k].stop - start)/(step*1.0)))
                if isinstance(step,types.FloatType) or \
                   isinstance(start, types.FloatType) or \
                   isinstance(key[k].stop, types.FloatType):
                       typecode = Numeric.Float
            if self.sparse:
                nn = map(lambda x,t: Numeric.arange(x,typecode=t),size,(typecode,)*len(size))
            else:
                nn = Numeric.indices(size,typecode)
	    for k in range(len(size)):
                step = key[k].step
                start = key[k].start
                if start is None: start=0
                if step is None: step=1
                if type(step) is type(1j):
                    step = int(abs(step))
                    step = (key[k].stop - start)/float(step-1)
                nn[k] = (nn[k]*step+start)
            if self.sparse:
                slobj = [Numeric.NewAxis]*len(size)
                for k in range(len(size)):
                    slobj[k] = slice(None,None)
                    nn[k] = nn[k][slobj]
                    slobj[k] = Numeric.NewAxis
	    return nn
        except (IndexError, TypeError):
            step = key.step
            stop = key.stop
            start = key.start
            if start is None: start = 0
            if type(step) is type(1j):
                step = abs(step)
                length = int(step)
                step = (key.stop-start)/float(step-1)
                stop = key.stop+step
                return Numeric.arange(0,length,1,Numeric.Float)*step + start
            else:
                return Numeric.arange(start, stop, step)
	    
    def __getslice__(self,i,j):
        return Numeric.arange(i,j)

    def __len__(self):
        return 0

mgrid = nd_grid()
ogrid = nd_grid(1)

import sys
class concatenator:
    """ Translates slice objects to concatenation along an axis.
    """
    def __init__(self, axis=0):
        self.axis = axis
    def __getitem__(self,key):
        if isinstance(key,types.StringType):
            frame = sys._getframe().f_back
            return array(matrix_base.bmat(key,frame.f_globals,frame.f_locals))
        if type(key) is not types.TupleType:
            key = (key,)
        objs = []
        for k in range(len(key)):
            if type(key[k]) is types.SliceType:
                typecode = Numeric.Int
	        step = key[k].step
                start = key[k].start
                stop = key[k].stop
                if start is None: start = 0
                if step is None:
                    step = 1
                if type(step) is type(1j):
                    size = int(abs(step))
                    typecode = Numeric.Float
                    endpoint = 1
                else:
                    size = int((stop - start)/(step*1.0))
                    endpoint = 0
                if isinstance(step,types.FloatType) or \
                   isinstance(start, types.FloatType) or \
                   isinstance(stop, types.FloatType):
                       typecode = Numeric.Float
                newobj = function_base.linspace(start, stop, num=size,
                                                endpoint=endpoint)
            elif type(key[k]) in ScalarType:
                newobj = asarray([key[k]])
            else:
                newobj = key[k]
            objs.append(newobj)
        return Numeric.concatenate(tuple(objs),axis=self.axis)
        
    def __getslice__(self,i,j):
        return Numeric.arange(i,j)

    def __len__(self):
        return 0

r_=concatenator(0)
c_=concatenator(-1)

# A nicer way to build up index tuples for arrays.
#
# You can do all this with slice() plus a few special objects,
# but there's a lot to remember. This version is simpler because
# it uses the standard array indexing syntax.
#
# Written by Konrad Hinsen <hinsen@cnrs-orleans.fr>
# last revision: 1999-7-23
#
# Cosmetic changes by T. Oliphant 2001
#
#
# This module provides a convenient method for constructing
# array indices algorithmically. It provides one importable object,
# 'index_expression'.
#
# For any index combination, including slicing and axis insertion,
# 'a[indices]' is the same as 'a[index_expression[indices]]' for any
# array 'a'. However, 'index_expression[indices]' can be used anywhere
# in Python code and returns a tuple of slice objects that can be
# used in the construction of complex index expressions.

class _index_expression_class:
    import sys
    maxint = sys.maxint

    def __getitem__(self, item):
        if type(item) != type(()):
            return (item,)
        else:
            return item

    def __len__(self):
        return self.maxint

    def __getslice__(self, start, stop):
        if stop == self.maxint:
            stop = None
        return self[start:stop:None]

index_exp = _index_expression_class()

# End contribution from Konrad.




# Copyright (c) 1996, 1997, The Regents of the University of California.
# All rights reserved.  See Legal.htm for full text and disclaimer.

from Numeric import *
from scipy_base.fastumath import *

GraphicsError = "GraphicsError"
import os
try:
   graphics = os.environ["PYGRAPH"]
except KeyError:
   graphics = "Gist"
 
if graphics [0:3] == "Nar" :
   import NarPlotter
elif graphics == "Gist" :
   import GistPlotter
else :
   raise GraphicsError , \
      graphics + " is an unknown graphics package. Check PYGRAPH " + \
         "environment variable."

if graphics [0:3] == "Nar" :
   print "This is a test of the Python interface to the Limeil Lab graphics"
   print "package, Narcisse. You  need Narcisse to be running. Fire it up by"
   print "doing setenv PORT_SERVEUR 0, typing  /dist/basis/Narcisse/bin/Narcisse &,"
   print "and then do another senetv PORT_SERVEUR to the port number which"
   print "appears in the Narcisse GUI."
else :
   print "This is a test of the Python interface to the Gist graphics package."

print "Invoke function demo () to run."

from plane import *
from surface import Surface
from graph3d import Graph3d
from mesh3d import *
from MLab import rand

def paws ( ) :
    i = raw_input ("Type in any string to continue; ^C to return to prompt. ")
    return

def demo () :
#  vsf = 0.
#  c = 1
   s = 1000.
   kmax = 25
   lmax = 35
   # The following computations define an interesting 3d surface.
   xr = multiply.outer (
      arange (1, kmax + 1, typecode = Float), ones (lmax, Float))
   yr = multiply.outer (
      ones (kmax, Float), arange (1, lmax + 1, typecode = Float))
   zt = 5. + xr + .2 * rand (kmax, lmax)   # ranf (xr)
   rt = 100. + yr + .2 * rand (kmax, lmax)   # ranf (yr)
   z = s * (rt + zt)
   z = z + .02 * z * rand (kmax, lmax)   # ranf (z)
   ut = rt / sqrt (rt ** 2 + zt ** 2)
   vt = zt / sqrt (rt ** 2 + zt ** 2)
   ireg =  multiply.outer ( ones (kmax), ones (lmax))
   ireg [0:1, 0:lmax] = 0
   ireg [0:kmax, 0:1] = 0
   ireg [1:15, 7:12] = 2
   ireg [1:15, 12:lmax] = 3
   ireg [3:7, 3:7] = 0
   freg = ireg + .2 * (1. - rand (kmax, lmax))  # ranf (ireg))
   freg = array (freg, Float)
   #rt [4:6, 4:6] = -1.e8
   z [3:10, 3:12] = z [3:10, 3:12] * .9
   z [5, 5] = z [5, 5] * .9
   z [17:22, 15:18] = z [17:22, 15:18] * 1.2
   z [16, 16] = z [16, 16] * 1.1
   
   s1 = Surface (z = z, mask = "max", opt_3d = ["wm", "i3"])
   g1 = Graph3d ( s1 , titles = "Surface with contour lines",
                  xyequal = 1., hardcopy = "talk.ps",
                  theta = 45., phi = 10., roll = 0.)
   g1.plot ( )
   paws ( )
   s1.set (opt_3d = ["wm", "f3"])
   g1.change (titles = "Flat mode")
   g1.plot ()
   paws ()
   s1.set (opt_3d = ["wm", "s3"])
   g1.change (titles = "Smooth mode")
   g1.plot ()
   paws ()
   s1.new (x = xr, y = yr, z = z - z [kmax/2, lmax/2],
           mask = "max", opt_3d = ["wm" , "i3"])
   s11 = sslice (s1, Plane (array ( [0., 0., 1.]), array ( [0., 0., 0.])))
   g1.replace (1, s11)
## g1.change ( titles = "Part of surface above xy plane", phi = 30.,
##          y_factor = 2.0,
##          axis_limits = [[0., 0.],[0., 0.], [0., 100000.]])
   g1.change ( titles = "Part of surface above xy plane",
            y_factor = 2.0,
            axis_limits = [[0., 0.],[0., 0.], [0., 100000.]])
   g1.plot ( )
   paws ( )
   s1.set (opt_3d = ["wm", "f3"])
   s11 = sslice (s1, Plane (array ( [0., 0., 1.]), array ( [0., 0., 0.])))
   g1.replace (1, s11)
   g1.change (titles = "Flat mode")
   g1.plot ()
   paws ()
   s1.set (opt_3d = ["wm", "s3"])
   s11 = sslice (s1, Plane (array ( [0., 0., 1.]), array ( [0., 0., 0.])))
   g1.replace (1, s11)
   g1.change (titles = "Smooth mode")
   g1.plot ()
   paws ()
   s1.set (z = z, c = freg, opt_3d = ["wm", "s4"])
## s1 = Surface (x = xr, y = yr, z = z, c = freg, opt_3d = ["wm", "s4"])
   g1.replace (1, s1)
   g1.change ( titles = "Surface colored by mesh values", phi = 20.,
           xyequal = 1,
           axis_limits = [[0., 0.], [0., 0.], [0., 0.], [0., 0.]])
   g1.plot ( )
   paws ( )
   xr1 = zeros ( (16, 7), Float)
   yr1 = zeros ( (16, 7), Float)
   z1 = zeros ( (16, 7), Float)
   zs1 = zeros ( (16, 7), Float)
   for i in range (16) :
       for j in range (7) :
           xr1 [i, j] = xr [i, j+6]
           yr1 [i, j] = yr [i, j+6]
           z1 [i, j] = z [i, j+6]
           zs1 [i, j] = freg [i, j+6]
   s1.set (x = xr1, y = yr1, z = z1, c = zs1)
   g1.change ( titles = "Region 2 colored by mesh values", phi = 10.)
   g1.plot ( )
   paws ( )
   s1.new (x = xr1, y = yr1, z = z1, opt_3d = ["wm", "s3"], mask = "max")
   g1.change ( titles = "Region 2 with mesh and contours", phi = 10.)
   g1.plot ( )
   paws ()
   s1.set (opt_3d = "s3")
   g1.change ( titles = "Region 2 with contours alone")
   g1.plot ()
   paws ( )
   zs1 = zeros ( (16, lmax - 7), Float)
   for i in range (16) :
       for j in range (lmax - 7) :
           zs1 [i, j] = z [i, j+6]
   s1.new (z = zs1, opt_3d = ["wm", "s3"], mask = "max")
   g1.change ( titles = "Regions 2 and 3, mesh and contours", 
            theta = 70., phi = 10., roll = 0.)
   g1.plot ( )
   paws ( )
   s1.set (opt_3d = "s3")
   g1.change ( titles = "Regions 2 and 3, contours alone")
   g1.plot ( )



# Copyright (c) 1996, 1997, The Regents of the University of California.
# All rights reserved.  See Legal.htm for full text and disclaimer.

from Numeric import *
from scipy_base.fastumath import *
from surface import *
from graph3d import *
def paws ( ) :
    i = raw_input ("Type in any string to continue; ^C to return to prompt. ")
    return
from PFB import *
f = PFB ('nicks_plot')
x = f.xout
y = f.yout
z = f.rhoout
s1 = Surface (x = y, y = x, z = z, opt_3d = "s3", mask = "max")
g1 = Graph3d (s1, theta = 11, phi = 45, titles = "Density plot: reflecting shock wave")
g1.plot ( )
paws ( )
g1.quick_plot (surface = 1, opt_3d = "w3")
paws ( )
vx = f.vxout
vy = f.vyout
massfrac = f.massfrac1out
 
f.close ()
s1.set (z = vx, mask = "max", opt_3d = "w3")
g1.change_plot (send = 1, theta = 160, phi = 45, titles = "x velocity: reflecting shock wave")
paws ()
s1.set (z = vy, mask = "max", opt_3d = "w3")
g1.change_plot (send = 1, theta = 11, phi = 45, titles = "y velocity: reflecting shock wave")
paws ()
s1.set (z = massfrac)
g1.change_plot (send = 1, theta = 170, phi = 45, titles = "Mass fraction: reflecting shock wave")
paws ()
f = PFB ('mikes_plot')
x8 = f.x8
y8 = f.y8
z8 = f.z8
s2 = Surface (x = x8, y = y8, z = z8, opt_3d = "s3", mask = "max",
     color_card = "redgreen")
g2 = Graph3d (s2, phi = 26, theta = 80, titles = "3-D square mode")
g2.plot ( )
f.close ( )



import inspect
import types
import sys

# NOTE:  pydoc defines a help function which works simliarly to this
#  except it uses a pager to take over the screen.

# combine name and arguments and split to multiple lines of
#  width characters.  End lines on a comma and begin argument list
#  indented with the rest of the arguments.
def split_line(name, arguments, width):
    firstwidth = len(name)
    k = firstwidth
    newstr = name
    sepstr = ", "
    arglist = arguments.split(sepstr)
    for argument in arglist:
        if k == firstwidth:
            addstr = ""
        else:
            addstr = sepstr
        k = k + len(argument) + len(addstr)
        if k > width:
            k = firstwidth + 1 + len(argument)
            newstr = newstr + ",\n" + " "*(firstwidth+2) + argument
        else:
            newstr = newstr + addstr + argument
    return newstr

_namedict = None
_dictlist = None

# Traverse all module directories underneath globals to see if something is defined
def makenamedict():
    import scipy
    thedict = {'scipy':scipy.__dict__}
    dictlist = ['scipy']
    totraverse = [scipy.__dict__]
    while 1:
        if len(totraverse) == 0:
            break
        thisdict = totraverse.pop(0)
        for x in thisdict.keys():
            if isinstance(thisdict[x],types.ModuleType):
                modname = thisdict[x].__name__
                if modname not in dictlist:
                    moddict = thisdict[x].__dict__
                    dictlist.append(modname)
                    totraverse.append(moddict)
                    thedict[modname] = moddict
    return thedict, dictlist


def info(object=None,maxwidth=76,output=sys.stdout,):
    """Get help information for a function, class, or module.
    
       Example:
          >>> from scipy import * 
          >>> info(polyval)
          polyval(p, x)
        
            Evaluate the polymnomial p at x.
            
            Description:        
                If p is of length N, this function returns the value:
                p[0]*(x**N-1) + p[1]*(x**N-2) + ... + p[N-2]*x + p[N-1]
    """
    global _namedict, _dictlist

    if hasattr(object,'_ppimport_importer') or \
       hasattr(object, '_ppimport_module'):
	object = object._ppimport_module
    elif hasattr(object, '_ppimport_attr'):
        object = object._ppimport_attr

    if object is None:        
        info(info)
    elif isinstance(object, types.StringType):
        if _namedict is None:
            _namedict, _dictlist = makenamedict()
        numfound = 0
        objlist = []
        for namestr in _dictlist:
            try:
                obj = _namedict[namestr][object]
                if id(obj) in objlist:
                    print >> output, "\n     *** Repeat reference found in %s *** " % namestr
                else:
                    objlist.append(id(obj))
                    print >> output, "     *** Found in %s ***" % namestr
                    info(obj)
                    print >> output, "-"*maxwidth
                numfound += 1
            except KeyError:
                pass
        if numfound == 0:
            print >> output, "Help for %s not found." % object
        else:
            print >> output, "\n     *** Total of %d references found. ***" % numfound
                    
    elif inspect.isfunction(object):
        name = object.func_name
        arguments = apply(inspect.formatargspec, inspect.getargspec(object))

        if len(name+arguments) > maxwidth:
            argstr = split_line(name, arguments, maxwidth)
        else:
            argstr = name + arguments

        print >> output, " " + argstr + "\n"
        print >> output, inspect.getdoc(object)

    elif inspect.isclass(object):  
        name = object.__name__
        if hasattr(object, '__init__'):
            arguments = apply(inspect.formatargspec, inspect.getargspec(object.__init__.im_func))        
            arglist = arguments.split(', ')
            if len(arglist) > 1:
                arglist[1] = "("+arglist[1]
                arguments = ", ".join(arglist[1:])
            else:
                arguments = "()"
        else:
            arguments = "()"

        if len(name+arguments) > maxwidth:
            argstr = split_line(name, arguments, maxwidth)
        else:
            argstr = name + arguments

        print >> output, " " + argstr + "\n"
        doc1 = inspect.getdoc(object) 
        if doc1 is None:
            if hasattr(object,'__init__'):
                print >> output, inspect.getdoc(object.__init__)
        else:
            print >> output, inspect.getdoc(object)

    elif type(object) is types.InstanceType: ## check for __call__ method
        print >> output, "Instance of class: ", object.__class__.__name__
        print >> output
        if hasattr(object, '__call__'):
            arguments = apply(inspect.formatargspec, inspect.getargspec(object.__call__.im_func))
            arglist = arguments.split(', ')
            if len(arglist) > 1:
                arglist[1] = "("+arglist[1]
                arguments = ", ".join(arglist[1:])
            else:
                arguments = "()"

            if hasattr(object,'name'):
		name = "%s" % object.name
            else:	
                name = "<name>"
            if len(name+arguments) > maxwidth:
                argstr = split_line(name, arguments, maxwidth)
            else:
                argstr = name + arguments

            print >> output, " " + argstr + "\n"
            doc = inspect.getdoc(object.__call__)
            if doc is not None:
                print >> output, inspect.getdoc(object.__call__)
            print >> output, inspect.getdoc(object)
            
        else:
            print >> output, inspect.getdoc(object)
                                            
    elif inspect.ismethod(object):
        name = object.__name__
        arguments = apply(inspect.formatargspec, inspect.getargspec(object.im_func))
        arglist = arguments.split(', ')
        if len(arglist) > 1:
            arglist[1] = "("+arglist[1]
            arguments = ", ".join(arglist[1:])
        else:
            arguments = "()"

        if len(name+arguments) > maxwidth:
            argstr = split_line(name, arguments, maxwidth)
        else:
            argstr = name + arguments

        print >> output, " " + argstr + "\n"
        print >> output, inspect.getdoc(object)

    elif hasattr(object, '__doc__'):
        print >> output, inspect.getdoc(object)
        

def source(object, output=sys.stdout):
    if inspect.isroutine(object):
        print >> output,  "In file: %s\n" % inspect.getsourcefile(object)
        print >> output,  inspect.getsource(object)
    else:
        print >> output,  "Not available for this object."




import os,sys,string
import pprint 

def remove_whitespace(in_str):
    import string
    out = string.replace(in_str," ","")
    out = string.replace(out,"\t","")
    out = string.replace(out,"\n","")
    return out
    
def print_assert_equal(test_string,actual,desired):
    """this should probably be in scipy_test.testing
    """
    try:
        assert(actual == desired)
    except AssertionError:
        import cStringIO
        msg = cStringIO.StringIO()
        msg.write(test_string)
        msg.write(' failed\nACTUAL: \n')
        pprint.pprint(actual,msg)
        msg.write('DESIRED: \n')
        pprint.pprint(desired,msg)
        raise AssertionError, msg.getvalue()

###################################################
# mainly used by catalog tests               
###################################################
from scipy_distutils.misc_util import add_grandparent_to_path,restore_path

add_grandparent_to_path(__name__)
import catalog
restore_path()

import glob

def temp_catalog_files(prefix=''):
    # might need to add some more platform specific catalog file
    # suffixes to remove.  The .pag was recently added for SunOS
    d = catalog.default_dir()
    f = catalog.os_dependent_catalog_name()
    return glob.glob(os.path.join(d,prefix+f+'*'))

import tempfile

def clear_temp_catalog():
    """ Remove any catalog from the temp dir
    """
    global backup_dir 
    backup_dir =tempfile.mktemp()
    os.mkdir(backup_dir)
    for file in temp_catalog_files():
        move_file(file,backup_dir)
        #d,f = os.path.split(file)
        #backup = os.path.join(backup_dir,f)
        #os.rename(file,backup)

def restore_temp_catalog():
    """ Remove any catalog from the temp dir
    """
    global backup_dir
    cat_dir = catalog.default_dir()
    for file in os.listdir(backup_dir):
        file = os.path.join(backup_dir,file)
        d,f = os.path.split(file)
        dst_file = os.path.join(cat_dir, f)
        if os.path.exists(dst_file):
            os.remove(dst_file)
        #os.rename(file,dst_file)
        move_file(file,dst_file)
    os.rmdir(backup_dir)
    backup_dir = None
         
def empty_temp_dir():
    """ Create a sub directory in the temp directory for use in tests
    """
    import tempfile
    d = catalog.default_dir()
    for i in range(10000):
        new_d = os.path.join(d,tempfile.gettempprefix()[1:-1]+`i`)
        if not os.path.exists(new_d):
            os.mkdir(new_d)
            break
    return new_d

def cleanup_temp_dir(d):
    """ Remove a directory created by empty_temp_dir
        should probably catch errors
    """
    files = map(lambda x,d=d: os.path.join(d,x),os.listdir(d))
    for i in files:
        try:
            if os.path.isdir(i):
                cleanup_temp_dir(i)
            else:
                os.remove(i)
        except OSError:
            pass # failed to remove file for whatever reason 
                 # (maybe it is a DLL Python is currently using)        
    try:
        os.rmdir(d)
    except OSError:
        pass        
        

# from distutils -- old versions had bug, so copying here to make sure 
# a working version is available.
from distutils.errors import DistutilsFileError
import distutils.file_util
def move_file (src, dst,
               verbose=0,
               dry_run=0):

    """Move a file 'src' to 'dst'.  If 'dst' is a directory, the file will
    be moved into it with the same name; otherwise, 'src' is just renamed
    to 'dst'.  Return the new full name of the file.

    Handles cross-device moves on Unix using 'copy_file()'.  What about
    other systems???
    """
    from os.path import exists, isfile, isdir, basename, dirname
    import errno

    if verbose:
        print "moving %s -> %s" % (src, dst)

    if dry_run:
        return dst

    if not isfile(src):
        raise DistutilsFileError, \
              "can't move '%s': not a regular file" % src

    if isdir(dst):
        dst = os.path.join(dst, basename(src))
    elif exists(dst):
        raise DistutilsFileError, \
              "can't move '%s': destination '%s' already exists" % \
              (src, dst)

    if not isdir(dirname(dst)):
        raise DistutilsFileError, \
              "can't move '%s': destination '%s' not a valid path" % \
              (src, dst)

    copy_it = 0
    try:
        os.rename(src, dst)
    except os.error, (num, msg):
        if num == errno.EXDEV:
            copy_it = 1
        else:
            raise DistutilsFileError, \
                  "couldn't move '%s' to '%s': %s" % (src, dst, msg)

    if copy_it:
        distutils.file_util.copy_file(src, dst)
        try:
            os.unlink(src)
        except os.error, (num, msg):
            try:
                os.unlink(dst)
            except os.error:
                pass
            raise DistutilsFileError, \
                  ("couldn't move '%s' to '%s' by copy/delete: " +
                   "delete '%s' failed: %s") % \
                  (src, dst, src, msg)

    return dst

        


#!/usr/bin/env python
#
# Created by: Pearu Peterson, March 2002
#
""" Test functions for linalg.basic module

"""
"""
Bugs:
1) solve.check_random_sym_complex fails if a is complex
   and transpose(a) = conjugate(a) (a is Hermitian).
"""
__usage__ = """
Build linalg:
  python setup_linalg.py build
Run tests if scipy is installed:
  python -c 'import scipy;scipy.linalg.test(<level>)'
Run tests if linalg is not installed:
  python tests/test_basic.py [<level>]
"""

import Numeric
from Numeric import arange, add, array, dot, zeros, identity

import sys
from scipy_test.testing import *
set_package_path()
from linalg import solve,inv,det,lstsq, toeplitz, hankel, tri, triu, tril
from linalg import pinv, pinv2, solve_banded
del sys.path[0]

import unittest

def random(size):
    return rand(*size)

def get_mat(n):
    data = arange(n)
    data = add.outer(data,data)
    return data

class test_solve_banded(ScipyTestCase):

    def check_simple(self):

        a = [[1,20,0,0],[-30,4,6,0],[2,1,20,2],[0,-1,7,14]]
        ab = [[0,20,6,2],
              [1,4,20,14],
              [-30,1,7,0],
              [2,-1,0,0]]
        l,u = 2,1
        for b in ([[1,0,0,0],[0,0,0,1],[0,1,0,0],[0,1,0,0]],
                  [[2,1],[-30,4],[2,3],[1,3]]):
            x = solve_banded((l,u),ab,b)
            assert_array_almost_equal(Numeric.matrixmultiply(a,x),b)

class test_solve(ScipyTestCase):

    def check_20Feb04_bug(self):
        a = [[1,1],[1.0,0]] # ok
        x0 = solve(a,[1,0j])
        assert_array_almost_equal(Numeric.matrixmultiply(a,x0),[1,0])

        a = [[1,1],[1.2,0]] # gives failure with clapack.zgesv(..,rowmajor=0)
        b = [1,0j]
        x0 = solve(a,b)
        assert_array_almost_equal(Numeric.matrixmultiply(a,x0),[1,0])

    def check_simple(self):
        a = [[1,20],[-30,4]]
        for b in ([[1,0],[0,1]],[1,0],
                  [[2,1],[-30,4]]):
            x = solve(a,b)
            assert_array_almost_equal(Numeric.matrixmultiply(a,x),b)
        
    def check_simple_sym(self):
        a = [[2,3],[3,5]]
        for lower in [0,1]:
            for b in ([[1,0],[0,1]],[1,0]):
                x = solve(a,b,sym_pos=1,lower=lower)
                assert_array_almost_equal(Numeric.matrixmultiply(a,x),b)

    def check_simple_sym_complex(self):
        a = [[5,2],[2,4]]
        for b in [[1j,0],
                  [[1j,1j],
                   [0,2]],
                  ]:
            x = solve(a,b,sym_pos=1)
            assert_array_almost_equal(Numeric.matrixmultiply(a,x),b)

    def check_simple_complex(self):
        a = array([[5,2],[2j,4]],'D')
        for b in [[1j,0],
                  [[1j,1j],
                   [0,2]],
                  [1,0j],
                  array([1,0],'D'),
                  ]:
            x = solve(a,b)
            assert_array_almost_equal(Numeric.matrixmultiply(a,x),b)

    def check_nils_20Feb04(self):
        n = 2
        A = random([n,n])+random([n,n])*1j
        X = zeros((n,n),'D')
        Ainv = inv(A) 
        R = identity(n)+identity(n)*0j
        for i in arange(0,n):
            r = R[:,i]
            X[:,i] = solve(A,r)
        assert_array_almost_equal(X,Ainv)

    def check_random(self):

        n = 20
        a = random([n,n])
        for i in range(n): a[i,i] = 20*(.1+a[i,i])
        for i in range(4):
            b = random([n,3])
            x = solve(a,b)
            assert_array_almost_equal(Numeric.matrixmultiply(a,x),b)

    def check_random_complex(self):
        n = 20
        a = random([n,n]) + 1j * random([n,n])
        for i in range(n): a[i,i] = 20*(.1+a[i,i])
        for i in range(2):
            b = random([n,3])
            x = solve(a,b)
            assert_array_almost_equal(Numeric.matrixmultiply(a,x),b)

    def check_random_sym(self):
        n = 20
        a = random([n,n])
        for i in range(n):
            a[i,i] = abs(20*(.1+a[i,i]))
            for j in range(i):
                a[i,j] = a[j,i]
        for i in range(4):
            b = random([n])
            x = solve(a,b,sym_pos=1)
            assert_array_almost_equal(Numeric.matrixmultiply(a,x),b)

    def check_random_sym_complex(self):
        n = 20
        a = random([n,n])
        #a  = a + 1j*random([n,n]) # XXX: with this the accuracy will be very low
        for i in range(n):
            a[i,i] = abs(20*(.1+a[i,i]))
            for j in range(i):
                a[i,j] = Numeric.conjugate(a[j,i])
        b = random([n])+2j*random([n])
        for i in range(2):
            x = solve(a,b,sym_pos=1)
            assert_array_almost_equal(Numeric.matrixmultiply(a,x),b)

    def bench_random(self,level=5):
        import LinearAlgebra
        Numeric_solve = LinearAlgebra.solve_linear_equations
        print
        print '      Solving system of linear equations'
        print '      =================================='

        print '      |    contiguous     |   non-contiguous '
        print '----------------------------------------------'
        print ' size |  scipy  | Numeric |  scipy  | Numeric'
        
        for size,repeat in [(20,1000),(100,150),(500,2),(1000,1)][:-1]:
            repeat *= 2
            print '%5s' % size,
            sys.stdout.flush()
            
            a = random([size,size])
            # larger diagonal ensures non-singularity:
            for i in range(size): a[i,i] = 10*(.1+a[i,i])
            b = random([size])

            print '| %6.2f ' % self.measure('solve(a,b)',repeat),
            sys.stdout.flush()

            print '| %6.2f ' % self.measure('Numeric_solve(a,b)',repeat),
            sys.stdout.flush()
                        
            a = a[-1::-1,-1::-1] # turn into a non-contiguous array
            assert not a.iscontiguous()

            print '| %6.2f ' % self.measure('solve(a,b)',repeat),
            sys.stdout.flush()

            print '| %6.2f ' % self.measure('Numeric_solve(a,b)',repeat),
            sys.stdout.flush()

            print '   (secs for %s calls)' % (repeat)

class test_inv(ScipyTestCase):

    def check_simple(self):
        a = [[1,2],[3,4]]
        a_inv = inv(a)
        assert_array_almost_equal(Numeric.matrixmultiply(a,a_inv),
                                  [[1,0],[0,1]])
        a = [[1,2,3],[4,5,6],[7,8,10]]
        a_inv = inv(a)
        assert_array_almost_equal(Numeric.matrixmultiply(a,a_inv),
                                  [[1,0,0],[0,1,0],[0,0,1]])

    def check_random(self):
        n = 20
        for i in range(4):
            a = random([n,n])
            for i in range(n): a[i,i] = 20*(.1+a[i,i])
            a_inv = inv(a)
            assert_array_almost_equal(Numeric.matrixmultiply(a,a_inv),
                                      Numeric.identity(n))
    def check_simple_complex(self):
        a = [[1,2],[3,4j]]
        a_inv = inv(a)
        assert_array_almost_equal(Numeric.matrixmultiply(a,a_inv),
                                  [[1,0],[0,1]])

    def check_random_complex(self):
        n = 20
        for i in range(4):
            a = random([n,n])+2j*random([n,n])
            for i in range(n): a[i,i] = 20*(.1+a[i,i])
            a_inv = inv(a)
            assert_array_almost_equal(Numeric.matrixmultiply(a,a_inv),
                                      Numeric.identity(n))

    def bench_random(self,level=5):
        import LinearAlgebra
        Numeric_inv = LinearAlgebra.inverse
        print
        print '           Finding matrix inverse'
        print '      =================================='
        print '      |    contiguous     |   non-contiguous '
        print '----------------------------------------------'
        print ' size |  scipy  | Numeric |  scipy  | Numeric'
        
        for size,repeat in [(20,1000),(100,150),(500,2),(1000,1)][:-1]:
            repeat *= 2
            print '%5s' % size,
            sys.stdout.flush()
            
            a = random([size,size])
            # large diagonal ensures non-singularity:
            for i in range(size): a[i,i] = 10*(.1+a[i,i])

            print '| %6.2f ' % self.measure('inv(a)',repeat),
            sys.stdout.flush()

            print '| %6.2f ' % self.measure('Numeric_inv(a)',repeat),
            sys.stdout.flush()
                        
            a = a[-1::-1,-1::-1] # turn into a non-contiguous array
            assert not a.iscontiguous()

            print '| %6.2f ' % self.measure('inv(a)',repeat),
            sys.stdout.flush()

            print '| %6.2f ' % self.measure('Numeric_inv(a)',repeat),
            sys.stdout.flush()

            print '   (secs for %s calls)' % (repeat)


class test_det(ScipyTestCase):

    def check_simple(self):
        a = [[1,2],[3,4]]
        a_det = det(a)
        assert_almost_equal(a_det,-2.0)

    def check_simple_complex(self):
        a = [[1,2],[3,4j]]
        a_det = det(a)
        assert_almost_equal(a_det,-6+4j)

    def check_random(self):
        import LinearAlgebra
        Numeric_det = LinearAlgebra.determinant
        n = 20
        for i in range(4):
            a = random([n,n])
            d1 = det(a)
            d2 = Numeric_det(a)
            assert_almost_equal(d1,d2)

    def check_random_complex(self):
        import LinearAlgebra
        Numeric_det = LinearAlgebra.determinant
        n = 20
        for i in range(4):
            a = random([n,n]) + 2j*random([n,n])
            d1 = det(a)
            d2 = Numeric_det(a)
            assert_almost_equal(d1,d2)

    def bench_random(self,level=5):
        import LinearAlgebra
        Numeric_det = LinearAlgebra.determinant
        print
        print '           Finding matrix determinant'
        print '      =================================='
        print '      |    contiguous     |   non-contiguous '
        print '----------------------------------------------'
        print ' size |  scipy  | Numeric |  scipy  | Numeric'
        
        for size,repeat in [(20,1000),(100,150),(500,2),(1000,1)][:-1]:
            repeat *= 2
            print '%5s' % size,
            sys.stdout.flush()
            
            a = random([size,size])

            print '| %6.2f ' % self.measure('det(a)',repeat),
            sys.stdout.flush()

            print '| %6.2f ' % self.measure('Numeric_det(a)',repeat),
            sys.stdout.flush()
                        
            a = a[-1::-1,-1::-1] # turn into a non-contiguous array
            assert not a.iscontiguous()

            print '| %6.2f ' % self.measure('det(a)',repeat),
            sys.stdout.flush()

            print '| %6.2f ' % self.measure('Numeric_det(a)',repeat),
            sys.stdout.flush()

            print '   (secs for %s calls)' % (repeat)


def direct_lstsq(a,b):
    a1 = Numeric.matrixmultiply(Numeric.transpose(a),a)
    b1 = Numeric.matrixmultiply(Numeric.transpose(a),b)
    return solve(a1,b1)

class test_lstsq(ScipyTestCase):

    def check_random_overdet_large(self):
        #bug report: Nils Wagner
        n = 200
        a = random([n,2])
        for i in range(2): a[i,i] = 20*(.1+a[i,i])
        b = random([n,3])
        x = lstsq(a,b)[0]
        assert_array_almost_equal(x,direct_lstsq(a,b))

    def check_simple_exact(self):
        a = [[1,20],[-30,4]]
        for b in ([[1,0],[0,1]],[1,0],
                  [[2,1],[-30,4]]):
            x = lstsq(a,b)[0]
            assert_array_almost_equal(Numeric.matrixmultiply(a,x),b)

    def check_simple_overdet(self):
        a = [[1,2],[4,5],[3,4]]
        b = [1,2,3]
        x,res,r,s = lstsq(a,b)
        #XXX: check defintion of res
        assert_array_almost_equal(x,direct_lstsq(a,b))

    def check_simple_underdet(self):
        a = [[1,2,3],[4,5,6]]
        b = [1,2]
        x,res,r,s = lstsq(a,b)
        #XXX: need independent check
        assert_array_almost_equal(x,[[-0.05555556],[0.11111111],[0.27777778]])

    def check_random_exact(self):

        n = 20
        a = random([n,n])
        for i in range(n): a[i,i] = 20*(.1+a[i,i])
        for i in range(4):
            b = random([n,3])
            x = lstsq(a,b)[0]
            assert_array_almost_equal(Numeric.matrixmultiply(a,x),b)

    def check_random_complex_exact(self):
        n = 20
        a = random([n,n]) + 1j * random([n,n])
        for i in range(n): a[i,i] = 20*(.1+a[i,i])
        for i in range(2):
            b = random([n,3])
            x = lstsq(a,b)[0]
            assert_array_almost_equal(Numeric.matrixmultiply(a,x),b)

    def check_random_overdet(self):
        n = 20
        m = 15
        a = random([n,m])
        for i in range(m): a[i,i] = 20*(.1+a[i,i])
        for i in range(4):
            b = random([n,3])
            x,res,r,s = lstsq(a,b)
            assert r==m,'unexpected efficient rank'
            #XXX: check definition of res
            assert_array_almost_equal(x,direct_lstsq(a,b))

    def check_random_complex_overdet(self):
        n = 20
        m = 15
        a = random([n,m]) + 1j * random([n,m])
        for i in range(m): a[i,i] = 20*(.1+a[i,i])
        for i in range(2):
            b = random([n,3])
            x,res,r,s = lstsq(a,b)
            assert r==m,'unexpected efficient rank'
            #XXX: check definition of res
            assert_array_almost_equal(x,direct_lstsq(a,b),1e-3)
            #XXX: tolerance 1e-3 is quite large, investigate the reason

class test_tri(unittest.TestCase):
    def check_basic(self):
        assert_equal(tri(4),array([[1,0,0,0],
                                   [1,1,0,0],
                                   [1,1,1,0],
                                   [1,1,1,1]]))
        assert_equal(tri(4,typecode='f'),array([[1,0,0,0],
                                                [1,1,0,0],
                                                [1,1,1,0],
                                                [1,1,1,1]],'f'))
    def check_diag(self):
        assert_equal(tri(4,k=1),array([[1,1,0,0],
                                       [1,1,1,0],
                                       [0,1,1,1],
                                       [1,1,1,1]]))
        assert_equal(tri(4,k=-1),array([[0,0,0,0],
                                        [1,0,0,0],
                                        [1,1,0,0],
                                        [1,1,1,0]]))
    def check_2d(self):
        assert_equal(tri(4,3),array([[1,0,0],
                                     [1,1,0],
                                     [1,1,1],
                                     [1,1,1]]))
        assert_equal(tri(3,4),array([[1,0,0,0],
                                     [1,1,0,0],
                                     [1,1,1,0]]))        
    def check_diag2d(self):
        assert_equal(tri(3,4,k=2),array([[1,1,1,0],
                                         [1,1,1,1],
                                         [1,1,1,1]]))
        assert_equal(tri(4,3,k=-2),array([[0,0,0],
                                          [0,0,0],
                                          [1,0,0],
                                          [1,1,0]]))

class test_tril(unittest.TestCase):
    def check_basic(self):
        a = (100*get_mat(5)).astype('l')
        b = a.copy()
        for k in range(5):
            for l in range(k+1,5):
                b[k,l] = 0
        assert_equal(tril(a),b)

    def check_diag(self):        
        a = (100*get_mat(5)).astype('f')
        b = a.copy()
        for k in range(5):
            for l in range(k+3,5):
                b[k,l] = 0
        assert_equal(tril(a,k=2),b)
        b = a.copy()
        for k in range(5):
            for l in range(max((k-1,0)),5):
                b[k,l] = 0
        assert_equal(tril(a,k=-2),b)

class test_triu(unittest.TestCase):
    def check_basic(self):
        a = (100*get_mat(5)).astype('l')
        b = a.copy()
        for k in range(5):
            for l in range(k+1,5):
                b[l,k] = 0
        assert_equal(triu(a),b)

    def check_diag(self):        
        a = (100*get_mat(5)).astype('f')
        b = a.copy()
        for k in range(5):
            for l in range(max((k-1,0)),5):
                b[l,k] = 0
        assert_equal(triu(a,k=2),b)
        b = a.copy()
        for k in range(5):
            for l in range(k+3,5):
                b[l,k] = 0
        assert_equal(tril(a,k=-2),b)

class test_toeplitz(unittest.TestCase):
    def check_basic(self):
        y = toeplitz([1,2,3])
        assert_array_equal(y,[[1,2,3],[2,1,2],[3,2,1]])
        y = toeplitz([1,2,3],[1,4,5])
        assert_array_equal(y,[[1,4,5],[2,1,4],[3,2,1]])
        
class test_hankel(unittest.TestCase):
    def check_basic(self):
        y = hankel([1,2,3])
        assert_array_equal(y,[[1,2,3],[2,3,0],[3,0,0]])
        y = hankel([1,2,3],[3,4,5])
        assert_array_equal(y,[[1,2,3],[2,3,4],[3,4,5]])

class test_pinv(ScipyTestCase):

    def check_simple(self):
        a=array([[1,2,3],[4,5,6.],[7,8,10]])
        a_pinv = pinv(a)
        assert_array_almost_equal(dot(a,a_pinv),[[1,0,0],[0,1,0],[0,0,1]])
        a_pinv = pinv2(a)
        assert_array_almost_equal(dot(a,a_pinv),[[1,0,0],[0,1,0],[0,0,1]])

    def check_simple_0det(self):
        a=array([[1,2,3],[4,5,6.],[7,8,9]])
        a_pinv = pinv(a)
        a_pinv2 = pinv2(a)
        assert_array_almost_equal(a_pinv,a_pinv2)

    def check_simple_cols(self):
        a=array([[1,2,3],[4,5,6.]])
        a_pinv = pinv(a)
        a_pinv2 = pinv2(a)
        assert_array_almost_equal(a_pinv,a_pinv2)

    def check_simple_rows(self):
        a=array([[1,2],[3,4],[5,6]])
        a_pinv = pinv(a)
        a_pinv2 = pinv2(a)
        assert_array_almost_equal(a_pinv,a_pinv2)

if __name__ == "__main__":
    ScipyTest('linalg.basic').run()



#
# special - Special Functions
#

from info_special import __doc__
from special_version import special_version as __version__

from basic import *
import specfun
import orthogonal
from orthogonal import legendre, chebyt, chebyu, chebyc, chebys, \
     jacobi, laguerre, genlaguerre, hermite, hermitenorm, gegenbauer, \
     sh_legendre, sh_chebyt, sh_chebyu, sh_jacobi, poch

